@@ -1,6 +1,6 @@
-/* 
+/*
  * FreeSWITCH Modular Media Switching Software Library / Soft-Switch Application
- * Copyright (C) 2005-2009, Anthony Minessale II <anthm@freeswitch.org>
+ * Copyright (C) 2005-2014, Anthony Minessale II <anthm@freeswitch.org>
  *
  * Version: MPL 1.1
  *
@@ -22,8 +22,10 @@
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
- * 
+ *
  * Anthony Minessale II <anthm@freeswitch.org>
+ * Luke Dashjr <luke@openmethods.com> (OpenMethods, LLC)
+ * Andrey Volk <andywolk@gmail.com>
  *
  *
  * switch_module_interfaces.h -- Module Interface Definitions
@@ -32,7 +34,7 @@
 /*! \file switch_module_interfaces.h
   \brief Module Interface Definitions
 
-  This module holds the definition of data abstractions used to implement various pluggable 
+  This module holds the definition of data abstractions used to implement various pluggable
   interfaces and pluggable event handlers.
 
 */
@@ -43,9 +45,9 @@
 #include "switch_resample.h"
 
 SWITCH_BEGIN_EXTERN_C
-/*! \brief A table of functions to execute at various states 
+/*! \brief A table of functions to execute at various states
  */
-typedef enum {
+	typedef enum {
 	SWITCH_SHN_ON_INIT,
 	SWITCH_SHN_ON_ROUTING,
 	SWITCH_SHN_ON_EXECUTE,
@@ -85,10 +87,12 @@ struct switch_state_handler_table {
 	switch_state_handler_t on_reporting;
 	/*! executed when the state changes to destroy */
 	switch_state_handler_t on_destroy;
+	int flags;
 	void *padding[10];
 };
 
 struct switch_stream_handle {
+	switch_stream_handle_read_function_t read_function;
 	switch_stream_handle_write_function_t write_function;
 	switch_stream_handle_raw_write_function_t raw_write_function;
 	void *data;
@@ -101,11 +105,11 @@ struct switch_stream_handle {
 };
 
 struct switch_io_event_hooks;
-
-
+struct switch_say_file_handle;
 
 typedef switch_call_cause_t (*switch_io_outgoing_channel_t)
-(switch_core_session_t *, switch_event_t *, switch_caller_profile_t *, switch_core_session_t **, switch_memory_pool_t **, switch_originate_flag_t);
+	(switch_core_session_t *, switch_event_t *, switch_caller_profile_t *, switch_core_session_t **, switch_memory_pool_t **, switch_originate_flag_t,
+	 switch_call_cause_t *);
 typedef switch_status_t (*switch_io_read_frame_t) (switch_core_session_t *, switch_frame_t **, switch_io_flag_t, int);
 typedef switch_status_t (*switch_io_write_frame_t) (switch_core_session_t *, switch_frame_t *, switch_io_flag_t, int);
 typedef switch_status_t (*switch_io_kill_channel_t) (switch_core_session_t *, int);
@@ -113,9 +117,12 @@ typedef switch_status_t (*switch_io_send_dtmf_t) (switch_core_session_t *, const
 typedef switch_status_t (*switch_io_receive_message_t) (switch_core_session_t *, switch_core_session_message_t *);
 typedef switch_status_t (*switch_io_receive_event_t) (switch_core_session_t *, switch_event_t *);
 typedef switch_status_t (*switch_io_state_change_t) (switch_core_session_t *);
+typedef switch_status_t (*switch_io_state_run_t) (switch_core_session_t *);
 typedef switch_status_t (*switch_io_read_video_frame_t) (switch_core_session_t *, switch_frame_t **, switch_io_flag_t, int);
 typedef switch_status_t (*switch_io_write_video_frame_t) (switch_core_session_t *, switch_frame_t *, switch_io_flag_t, int);
-typedef switch_call_cause_t (*switch_io_resurrect_session_t) (switch_core_session_t **, switch_memory_pool_t **, void *);
+typedef switch_status_t (*switch_io_read_text_frame_t) (switch_core_session_t *, switch_frame_t **, switch_io_flag_t, int);
+typedef switch_status_t (*switch_io_write_text_frame_t) (switch_core_session_t *, switch_frame_t *, switch_io_flag_t, int);
+typedef switch_jb_t *(*switch_io_get_jb_t) (switch_core_session_t *, switch_media_type_t);
 
 typedef enum {
 	SWITCH_IO_OUTGOING_CHANNEL,
@@ -128,7 +135,9 @@ typedef enum {
 	SWITCH_IO_STATE_CHANGE,
 	SWITCH_IO_READ_VIDEO_FRAME,
 	SWITCH_IO_WRITE_VIDEO_FRAME,
-	SWITCH_IO_RESURRECT_SESSION
+	SWITCH_IO_READ_TEXT_FRAME,
+	SWITCH_IO_WRITE_TEXT_FRAME,
+	SWITCH_IO_GET_JB,
 } switch_io_routine_name_t;
 
 /*! \brief A table of i/o routines that an endpoint interface can implement */
@@ -153,8 +162,14 @@ struct switch_io_routines {
 	switch_io_read_video_frame_t read_video_frame;
 	/*! write a video frame to a session */
 	switch_io_write_video_frame_t write_video_frame;
-	/*! resurrect a session */
-	switch_io_resurrect_session_t resurrect_session;
+	/*! read a video frame from a session */
+	switch_io_read_text_frame_t read_text_frame;
+	/*! write a video frame to a session */
+	switch_io_write_text_frame_t write_text_frame;
+	/*! change a sessions channel run state */
+	switch_io_state_run_t state_run;
+	/*! get sessions jitterbuffer */
+	switch_io_get_jb_t get_jb;
 	void *padding[10];
 };
 
@@ -183,8 +198,12 @@ struct switch_endpoint_interface {
 
 	/* parent */
 	switch_loadable_module_interface_t *parent;
+
 	/* to facilitate linking */
 	struct switch_endpoint_interface *next;
+
+	switch_core_recover_callback_t recover_callback;
+
 };
 
 /*! \brief Abstract handler to a timer module */
@@ -197,6 +216,7 @@ struct switch_timer {
 	unsigned int samples;
 	/*! current sample count based on samples parameter */
 	uint32_t samplecount;
+	uint32_t last_samplecount;
 	/*! the timer interface provided from a loadable module */
 	switch_timer_interface_t *timer_interface;
 	/*! the timer's memory pool */
@@ -205,7 +225,9 @@ struct switch_timer {
 	void *private_info;
 	/*! remaining time from last call to _check() */
 	switch_size_t diff;
-	switch_size_t tick;
+	switch_time_t start;
+	uint64_t tick;
+
 };
 
 typedef enum {
@@ -261,16 +283,26 @@ struct switch_file_interface {
 	switch_status_t (*file_open) (switch_file_handle_t *, const char *file_path);
 	/*! function to close the file */
 	switch_status_t (*file_close) (switch_file_handle_t *);
+	/*! function to close the file */
+	switch_status_t (*file_truncate) (switch_file_handle_t *, int64_t offset);
 	/*! function to read from the file */
 	switch_status_t (*file_read) (switch_file_handle_t *, void *data, switch_size_t *len);
 	/*! function to write from the file */
 	switch_status_t (*file_write) (switch_file_handle_t *, void *data, switch_size_t *len);
 	/*! function to seek to a certian position in the file */
+	switch_status_t (*file_read_video) (switch_file_handle_t *, switch_frame_t *frame, switch_video_read_flag_t flags);
+	/*! function to write from the file */
+	switch_status_t (*file_write_video) (switch_file_handle_t *, switch_frame_t *frame);
+	/*! function to seek to a certian position in the file */
 	switch_status_t (*file_seek) (switch_file_handle_t *, unsigned int *cur_pos, int64_t samples, int whence);
 	/*! function to set meta data */
 	switch_status_t (*file_set_string) (switch_file_handle_t *fh, switch_audio_col_t col, const char *string);
 	/*! function to get meta data */
 	switch_status_t (*file_get_string) (switch_file_handle_t *fh, switch_audio_col_t col, const char **string);
+	/*! function to pre close the file to read params */
+	switch_status_t (*file_pre_close) (switch_file_handle_t *fh);
+	/*! function to control the underlying tech of the file  */
+	switch_status_t (*file_command) (switch_file_handle_t *fh, switch_file_command_t command);
 	/*! list of supported file extensions */
 	char **extens;
 	switch_thread_rwlock_t *rwlock;
@@ -295,7 +327,8 @@ struct switch_file_handle {
 	/*! the current native samplerate */
 	uint32_t native_rate;
 	/*! the number of channels */
-	uint8_t channels;
+	uint32_t channels;
+	uint32_t real_channels;
 	/*! integer representation of the format */
 	unsigned int format;
 	/*! integer representation of the sections */
@@ -323,6 +356,7 @@ struct switch_file_handle {
 	switch_size_t samples_in;
 	switch_size_t samples_out;
 	int32_t vol;
+	int32_t volgranular;
 	switch_audio_resampler_t *resampler;
 	switch_buffer_t *buffer;
 	switch_byte_t *dbuf;
@@ -333,6 +367,23 @@ struct switch_file_handle {
 	const char *file;
 	const char *func;
 	int line;
+	char *file_path;
+	char *spool_path;
+	const char *prefix;
+	int max_samples;
+	switch_event_t *params;
+	uint32_t cur_channels;
+	uint32_t cur_samplerate;
+	char *stream_name;
+	char *modname;
+	switch_mm_t mm;
+	switch_mutex_t *flag_mutex;
+	/*! total video duration, or total page in pdf*/
+	int64_t duration;
+	/*! current video position, or current page in pdf */
+	int64_t vpos;
+	void *muxbuf;
+	switch_size_t muxlen;
 };
 
 /*! \brief Abstract interface to an asr module */
@@ -357,6 +408,8 @@ struct switch_asr_interface {
 	switch_status_t (*asr_check_results) (switch_asr_handle_t *ah, switch_asr_flag_t *flags);
 	/*! function to read results from the ASR */
 	switch_status_t (*asr_get_results) (switch_asr_handle_t *ah, char **xmlstr, switch_asr_flag_t *flags);
+	/*! function to read result headers from the ASR */
+	switch_status_t (*asr_get_result_headers) (switch_asr_handle_t *ah, switch_event_t **headers, switch_asr_flag_t *flags);
 	/*! function to start ASR input timers */
 	switch_status_t (*asr_start_input_timers) (switch_asr_handle_t *ah);
 	void (*asr_text_param) (switch_asr_handle_t *ah, char *param, const char *val);
@@ -367,6 +420,14 @@ struct switch_asr_interface {
 	switch_mutex_t *reflock;
 	switch_loadable_module_interface_t *parent;
 	struct switch_asr_interface *next;
+	/*! function to enable a grammar to the asr interface */
+	switch_status_t (*asr_enable_grammar) (switch_asr_handle_t *ah, const char *name);
+	/*! function to disable a grammar to the asr interface */
+	switch_status_t (*asr_disable_grammar) (switch_asr_handle_t *ah, const char *name);
+	/*! function to disable all grammars to the asr interface */
+	switch_status_t (*asr_disable_all_grammars) (switch_asr_handle_t *ah);
+	/*! function to feed DTMF to the ASR */
+	switch_status_t (*asr_feed_dtmf) (switch_asr_handle_t *ah, const switch_dtmf_t *dtmf, switch_asr_flag_t *flags);
 };
 
 /*! an abstract representation of an asr speech interface. */
@@ -386,6 +447,14 @@ struct switch_asr_handle {
 	char *param;
 	/*! the handle's memory pool */
 	switch_memory_pool_t *memory_pool;
+	switch_buffer_t *buffer;
+	switch_byte_t *dbuf;
+	switch_size_t dbuflen;
+	switch_audio_resampler_t *resampler;
+	/*! the current samplerate */
+	uint32_t samplerate;
+	/*! the current native samplerate */
+	uint32_t native_rate;
 	/*! private data for the format module to store handle specific info */
 	void *private_info;
 };
@@ -395,7 +464,7 @@ struct switch_speech_interface {
 	/*! the name of the interface */
 	const char *interface_name;
 	/*! function to open the speech interface */
-	switch_status_t (*speech_open) (switch_speech_handle_t *sh, const char *voice_name, int rate, switch_speech_flag_t *flags);
+	switch_status_t (*speech_open) (switch_speech_handle_t *sh, const char *voice_name, int rate, int channels, switch_speech_flag_t *flags);
 	/*! function to close the speech interface */
 	switch_status_t (*speech_close) (switch_speech_handle_t *, switch_speech_flag_t *flags);
 	/*! function to feed audio to the ASR */
@@ -426,6 +495,8 @@ struct switch_speech_handle {
 	uint32_t rate;
 	uint32_t speed;
 	uint32_t samples;
+	uint32_t channels;
+	uint32_t real_channels;
 	char voice[80];
 	char *engine;
 	/*! module specific param */
@@ -435,7 +506,7 @@ struct switch_speech_handle {
 	switch_audio_resampler_t *resampler;
 	switch_buffer_t *buffer;
 	switch_byte_t *dbuf;
-	switch_size_t dbuflen;	
+	switch_size_t dbuflen;
 	/*! the current samplerate */
 	uint32_t samplerate;
 	/*! the current native samplerate */
@@ -452,6 +523,7 @@ struct switch_say_interface {
 	const char *interface_name;
 	/*! function to pass down to the module */
 	switch_say_callback_t say_function;
+	switch_say_string_callback_t say_string_function;
 	switch_thread_rwlock_t *rwlock;
 	int refs;
 	switch_mutex_t *reflock;
@@ -464,8 +536,8 @@ struct switch_chat_interface {
 	/*! the name of the interface */
 	const char *interface_name;
 	/*! function to open the directory interface */
-	switch_status_t (*chat_send) (const char *proto, const char *from, const char *to, 
-								  const char *subject, const char *body, const char *type, const char *hint);
+	switch_status_t (*chat_send) (switch_event_t *message_event);
+
 	switch_thread_rwlock_t *rwlock;
 	int refs;
 	switch_mutex_t *reflock;
@@ -486,6 +558,30 @@ struct switch_management_interface {
 	struct switch_management_interface *next;
 };
 
+/*! \brief Abstract interface to a limit module */
+struct switch_limit_interface {
+	/*! name of the interface */
+	const char *interface_name;
+	/*! increment */
+	switch_status_t (*incr) (switch_core_session_t *session, const char *realm, const char *resource, const int max, const int interval);
+	/*! release */
+	switch_status_t (*release) (switch_core_session_t *session, const char *realm, const char *resource);
+	/*! usage for resource */
+	int (*usage) (const char *realm, const char *resource, uint32_t *rcount);
+	/*! reset counters */
+	switch_status_t (*reset) (void);
+	/*! freform status */
+	char * (*status) (void);
+	/*! reset interval counter */
+	switch_status_t (*interval_reset) (const char *realm, const char *resource);
+	/* internal */
+	switch_thread_rwlock_t *rwlock;
+	int refs;
+	switch_mutex_t *reflock;
+	switch_loadable_module_interface_t *parent;
+	struct switch_limit_interface *next;
+};
+
 /*! \brief Abstract interface to a directory module */
 struct switch_directory_interface {
 	/*! the name of the interface */
@@ -520,42 +616,84 @@ struct switch_directory_handle {
 	void *private_info;
 };
 
+/*! \brief Abstract interface to a database module */
+struct switch_database_interface {
+	/*! the name of the interface */
+	const char *interface_name;
+	/*! flags indicating database specifics, see switch_database_flag_t */
+	uint32_t flags;
+	switch_status_t(*handle_new)(switch_cache_db_database_interface_options_t database_interface_options, switch_database_interface_handle_t **dih);
+	switch_status_t(*handle_destroy)(switch_database_interface_handle_t **dih); 
+	switch_status_t(*flush)(switch_database_interface_handle_t *dih);
+	switch_status_t(*exec_detailed)(const char *file, const char *func, int line, 
+		switch_database_interface_handle_t *dih, const char *sql, char **err);
+	switch_status_t(*exec_string)(switch_database_interface_handle_t *dih, const char *sql, char *resbuf, size_t len, char **err);
+	switch_status_t(*sql_set_auto_commit_attr)(switch_database_interface_handle_t *dih, switch_bool_t on);
+	switch_status_t(*commit)(switch_database_interface_handle_t *dih);
+	switch_status_t(*rollback)(switch_database_interface_handle_t *dih);
+	switch_status_t(*callback_exec_detailed)(const char *file, const char *func, int line,
+		switch_database_interface_handle_t *dih, const char *sql, switch_core_db_callback_func_t callback, void *pdata, char **err);
+	switch_status_t(*affected_rows)(switch_database_interface_handle_t *dih, int *affected_rows);
 
-/* nobody has more setting than speex so we will let them set the standard */
-/*! \brief Various codec settings (currently only relevant to speex) */
-struct switch_codec_settings {
-	/*! desired quality */
-	int quality;
-	/*! desired complexity */
-	int complexity;
-	/*! desired enhancement */
-	int enhancement;
-	/*! desired vad level */
-	int vad;
-	/*! desired vbr level */
-	int vbr;
-	/*! desired vbr quality */
-	float vbr_quality;
-	/*! desired abr level */
-	int abr;
-	/*! desired dtx setting */
-	int dtx;
-	/*! desired preprocessor settings */
-	int preproc;
-	/*! preprocessor vad settings */
-	int pp_vad;
-	/*! preprocessor gain control settings */
-	int pp_agc;
-	/*! preprocessor gain level */
-	float pp_agc_level;
-	/*! preprocessor denoise level */
-	int pp_denoise;
-	/*! preprocessor dereverb settings */
-	int pp_dereverb;
-	/*! preprocessor dereverb decay level */
-	float pp_dereverb_decay;
-	/*! preprocessor dereverb level */
-	float pp_dereverb_level;
+	/*! list of supported dsn prefixes */
+	char **prefixes;
+	switch_thread_rwlock_t *rwlock;
+	int refs;
+	switch_mutex_t *reflock;
+	switch_loadable_module_interface_t *parent;
+	struct switch_database_interface *next;
+};
+
+/*! an abstract representation of a database interface. */
+struct switch_database_interface_handle {
+	switch_cache_db_database_interface_options_t connection_options;
+	void *handle;
+};
+
+struct switch_audio_codec_settings {
+	int unused;
+};
+
+struct switch_video_codec_settings {
+	uint32_t bandwidth;
+	int32_t width;
+	int32_t height;
+	uint8_t try_hardware_encoder;
+	uint8_t fps;
+	char config_profile_name[64];
+};
+
+union switch_codec_settings {
+	struct switch_audio_codec_settings audio;
+	struct switch_video_codec_settings video;
+};
+
+/*! an abstract handle of a fmtp parsed by codec */
+struct switch_codec_fmtp {
+	/*! actual samples transferred per second for those who are not moron g722 RFC writers */
+	uint32_t actual_samples_per_second;
+	/*! bits transferred per second */
+	int bits_per_second;
+	/*! number of microseconds of media in one packet (ptime * 1000) */
+	int microseconds_per_packet;
+	/*! maximum ptime in ms */
+	int max_ptime;
+	/*! minimum ptime in ms */
+	int min_ptime;
+	/*! stereo, typically bidirectional */
+	int stereo; 
+	/* sender properties (stereo) */
+	int sprop_stereo;
+	/*! private data for the codec module to store handle specific info */
+	void *private_info;
+
+};
+
+struct switch_picture {
+	uint32_t width;      /* the picture width */
+	uint32_t height;     /* the picture height */
+	uint8_t *planes[4];  /* pointer to the top left pixel for each plane */
+	uint32_t stride[4];  /* stride between rows for each plane */
 };
 
 /*! an abstract handle to a codec module */
@@ -568,8 +706,6 @@ struct switch_codec {
 	char *fmtp_in;
 	/*! fmtp line for local sdp */
 	char *fmtp_out;
-	/*! codec settings for this handle */
-	switch_codec_settings_t codec_settings;
 	/*! flags to modify behaviour */
 	uint32_t flags;
 	/*! the handle's memory pool */
@@ -578,6 +714,9 @@ struct switch_codec {
 	void *private_info;
 	switch_payload_t agreed_pt;
 	switch_mutex_t *mutex;
+	struct switch_codec *next;
+	switch_core_session_t *session;
+	switch_frame_t *cur_frame;
 };
 
 /*! \brief A table of settings and callbacks that define a paticular implementation of a codec */
@@ -614,10 +753,17 @@ struct switch_codec_implementation {
 	switch_core_codec_encode_func_t encode;
 	/*! function to decode encoded data into raw data */
 	switch_core_codec_decode_func_t decode;
+	/*! function to encode video raw data into encoded data */
+	switch_core_codec_video_encode_func_t encode_video;
+	/*! function to decode video encoded data into raw data */
+	switch_core_codec_video_decode_func_t decode_video;
+	/*! function to send control messages to the codec */
+	switch_core_codec_control_func_t codec_control;
 	/*! deinitalize a codec handle using this implementation */
 	switch_core_codec_destroy_func_t destroy;
 	uint32_t codec_id;
 	uint32_t impl_id;
+	char *modname;
 	struct switch_codec_implementation *next;
 };
 
@@ -627,10 +773,13 @@ struct switch_codec_interface {
 	const char *interface_name;
 	/*! a list of codec implementations related to the codec */
 	switch_codec_implementation_t *implementations;
+	/*! function to decode a codec fmtp parameters */
+	switch_core_codec_fmtp_parse_func_t parse_fmtp;
 	uint32_t codec_id;
 	switch_thread_rwlock_t *rwlock;
 	int refs;
 	switch_mutex_t *reflock;
+	char *modname;
 	switch_loadable_module_interface_t *parent;
 	struct switch_codec_interface *next;
 };
@@ -656,6 +805,27 @@ struct switch_application_interface {
 	struct switch_application_interface *next;
 };
 
+/*! \brief A module interface to implement a chat application */
+struct switch_chat_application_interface {
+	/*! the name of the interface */
+	const char *interface_name;
+	/*! function the application implements */
+	switch_chat_application_function_t chat_application_function;
+	/*! the long winded description of the application */
+	const char *long_desc;
+	/*! the short and sweet description of the application */
+	const char *short_desc;
+	/*! an example of the application syntax */
+	const char *syntax;
+	/*! flags to control behaviour */
+	uint32_t flags;
+	switch_thread_rwlock_t *rwlock;
+	int refs;
+	switch_mutex_t *reflock;
+	switch_loadable_module_interface_t *parent;
+	struct switch_chat_application_interface *next;
+};
+
 /*! \brief A module interface to implement an api function */
 struct switch_api_interface {
 	/*! the name of the interface */
@@ -673,9 +843,35 @@ struct switch_api_interface {
 	struct switch_api_interface *next;
 };
 
-#define PROTECT_INTERFACE(_it) if (_it) {switch_mutex_lock(_it->reflock); switch_thread_rwlock_rdlock(_it->parent->rwlock); switch_thread_rwlock_rdlock(_it->rwlock); _it->refs++; _it->parent->refs++; switch_mutex_unlock(_it->reflock);} //if (!strcmp(_it->interface_name, "user")) switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_CRIT, "+++++++++++LOCK %s %d/%d\n", _it->interface_name, _it->refs, _it->parent->refs);
-#define UNPROTECT_INTERFACE(_it) if (_it) {switch_mutex_lock(_it->reflock); switch_thread_rwlock_unlock(_it->rwlock); switch_thread_rwlock_unlock(_it->parent->rwlock); _it->refs--; _it->parent->refs--; switch_mutex_unlock(_it->reflock);} //if (!strcmp(_it->interface_name, "user")) switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_CRIT, "---------UNLOCK %s %d/%d\n", _it->interface_name, _it->refs, _it->parent->refs);
 
+/*! \brief A module interface to implement a json api function */
+struct switch_json_api_interface {
+	/*! the name of the interface */
+	const char *interface_name;
+	/*! a description of the api function */
+	const char *desc;
+	/*! function the api call uses */
+	switch_json_api_function_t function;
+	/*! an example of the api syntax */
+	const char *syntax;
+	switch_thread_rwlock_t *rwlock;
+	int refs;
+	switch_mutex_t *reflock;
+	switch_loadable_module_interface_t *parent;
+	struct switch_json_api_interface *next;
+};
+
+#define PROTECT_INTERFACE(_it) if (_it) {switch_thread_rwlock_rdlock(_it->parent->rwlock); switch_thread_rwlock_rdlock(_it->rwlock); switch_mutex_lock(_it->reflock); _it->refs++; _it->parent->refs++; switch_mutex_unlock(_it->reflock);}	//if (!strcmp(_it->interface_name, "user")) switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_CRIT, "+++++++++++LOCK %s %d/%d\n", _it->interface_name, _it->refs, _it->parent->refs);
+#define UNPROTECT_INTERFACE(_it) if (_it) {switch_mutex_lock(_it->reflock); _it->refs--; _it->parent->refs--; switch_mutex_unlock(_it->reflock); switch_thread_rwlock_unlock(_it->rwlock); switch_thread_rwlock_unlock(_it->parent->rwlock);}	//if (!strcmp(_it->interface_name, "user")) switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_CRIT, "---------UNLOCK %s %d/%d\n", _it->interface_name, _it->refs, _it->parent->refs);
+
+#include "switch_frame.h"
+
+struct switch_slin_data {
+	switch_core_session_t *session;
+	switch_frame_t write_frame;
+	switch_codec_t codec;
+	char frame_data[SWITCH_RECOMMENDED_BUFFER_SIZE];
+};
 
 SWITCH_END_EXTERN_C
 #endif
@@ -687,5 +883,5 @@ SWITCH_END_EXTERN_C
  * c-basic-offset:4
  * End:
  * For VIM:
- * vim:set softtabstop=4 shiftwidth=4 tabstop=4:
+ * vim:set softtabstop=4 shiftwidth=4 tabstop=4 noet:
  */
