@@ -1,6 +1,6 @@
-/* 
+/*
  * FreeSWITCH Modular Media Switching Software Library / Soft-Switch Application
- * Copyright (C) 2005-2010, Anthony Minessale II <anthm@freeswitch.org>
+ * Copyright (C) 2005-2014, Anthony Minessale II <anthm@freeswitch.org>
  *
  * Version: MPL 1.1
  *
@@ -22,7 +22,7 @@
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
- * 
+ *
  * Anthony Minessale II <anthm@freeswitch.org>
  *
  *
@@ -34,8 +34,8 @@
   \ingroup core1
   \{
 */
-#ifndef _SWITCH_STUN_PARSER_H
-#define _SWITCH_STUN_PARSER_H
+#ifndef FREESWITCH_STUN_PARSER_H
+#define FREESWITCH_STUN_PARSER_H
 
 SWITCH_BEGIN_EXTERN_C
 #define SWITCH_STUN_DEFAULT_PORT 3478
@@ -57,6 +57,8 @@ SWITCH_BEGIN_EXTERN_C
 	SWITCH_STUN_DATA_INDICATION = 0x0115
 } switch_stun_message_t;
 
+#define STUN_MAGIC_COOKIE 0x2112A442
+
 typedef enum {
 	SWITCH_STUN_ATTR_MAPPED_ADDRESS = 0x0001,	/* Address */
 	SWITCH_STUN_ATTR_RESPONSE_ADDRESS = 0x0002,	/* Address */
@@ -77,7 +79,23 @@ typedef enum {
 	SWITCH_STUN_ATTR_DESTINATION_ADDRESS = 0x0011,	/* Address */
 	SWITCH_STUN_ATTR_SOURCE_ADDRESS2 = 0x0012,	/* Address */
 	SWITCH_STUN_ATTR_DATA = 0x0013,	/* ByteString */
-	SWITCH_STUN_ATTR_OPTIONS = 0x8001	/* UInt32 */
+	SWITCH_STUN_ATTR_OPTIONS = 0x8001,	/* UInt32 */
+	SWITCH_STUN_ATTR_XOR_MAPPED_ADDRESS = 0x0020,   /* Address */
+
+	/* ice crap */
+
+	SWITCH_STUN_ATTR_PRIORITY           = 0x0024,
+	SWITCH_STUN_ATTR_USE_CAND           = 0x0025,
+	SWITCH_STUN_ATTR_PADDING            = 0x0026,
+	SWITCH_STUN_ATTR_RESP_PORT          = 0x0027,
+	SWITCH_STUN_ATTR_SOFTWARE           = 0x8022,
+	SWITCH_STUN_ATTR_ALT_SERVER         = 0x8023,
+	SWITCH_STUN_ATTR_FINGERPRINT        = 0x8028,
+	SWITCH_STUN_ATTR_CONTROLLED         = 0x8029,
+	SWITCH_STUN_ATTR_CONTROLLING        = 0x802a,
+	SWITCH_STUN_ATTR_RESP_ORIGIN        = 0x802b,
+	SWITCH_STUN_ATTR_OTHER_ADDR         = 0x802c
+
 } switch_stun_attribute_t;
 
 typedef enum {
@@ -101,7 +119,8 @@ typedef enum {
 typedef struct {
 	uint16_t type;
 	uint16_t length;
-	char id[16];
+	uint32_t cookie;
+	char id[12];
 } switch_stun_packet_header_t;
 
 typedef struct {
@@ -122,6 +141,33 @@ typedef struct {
 	uint32_t address;
 } switch_stun_ip_t;
 
+typedef struct {
+	uint8_t wasted;
+	uint8_t family;
+	uint16_t port;
+	uint8_t address[16];
+} switch_stun_ipv6_t;
+
+#if SWITCH_BYTE_ORDER == __BIG_ENDIAN
+
+typedef struct {
+	unsigned padding:21;
+	unsigned code:3;
+	unsigned number:8;
+	char reason[764];
+} switch_stun_error_code_t;
+
+#else
+
+typedef struct {
+	unsigned number:8;
+	unsigned code:3;
+	unsigned padding:21;
+	char reason[764];
+} switch_stun_error_code_t;
+
+#endif
+
 
 /*!
   \brief Writes random characters into a buffer
@@ -156,7 +202,9 @@ SWITCH_DECLARE(char *) switch_stun_host_lookup(const char *host, switch_memory_p
   \param port the port
   \return true or false
 */
-SWITCH_DECLARE(uint8_t) switch_stun_packet_attribute_get_mapped_address(switch_stun_packet_attribute_t *attribute, char *ipstr, uint16_t *port);
+SWITCH_DECLARE(uint8_t) switch_stun_packet_attribute_get_mapped_address(switch_stun_packet_attribute_t *attribute, char *ipstr, switch_size_t iplen, uint16_t *port);
+SWITCH_DECLARE(uint8_t) switch_stun_packet_attribute_get_xor_mapped_address(switch_stun_packet_attribute_t *attribute, switch_stun_packet_header_t *header, char *ipstr, switch_size_t iplen, uint16_t *port);
+
 
 /*!
   \brief Extract a username from a packet attribute
@@ -185,7 +233,8 @@ SWITCH_DECLARE(switch_stun_packet_t *) switch_stun_packet_build_header(switch_st
   \return true or false
 */
 SWITCH_DECLARE(uint8_t) switch_stun_packet_attribute_add_username(switch_stun_packet_t *packet, char *username, uint16_t ulen);
-
+SWITCH_DECLARE(uint8_t) switch_stun_packet_attribute_add_password(switch_stun_packet_t *packet, char *password, uint16_t ulen);
+SWITCH_DECLARE(uint8_t) switch_stun_packet_attribute_add_software(switch_stun_packet_t *packet, char *software, uint16_t ulen);
 
 /*!
   \brief Add a binded address packet attribute
@@ -194,7 +243,15 @@ SWITCH_DECLARE(uint8_t) switch_stun_packet_attribute_add_username(switch_stun_pa
   \param port the port of the mapped address
   \return true or false
 */
-SWITCH_DECLARE(uint8_t) switch_stun_packet_attribute_add_binded_address(switch_stun_packet_t *packet, char *ipstr, uint16_t port);
+SWITCH_DECLARE(uint8_t) switch_stun_packet_attribute_add_binded_address(switch_stun_packet_t *packet, char *ipstr, uint16_t port, int family);
+SWITCH_DECLARE(uint8_t) switch_stun_packet_attribute_add_xor_binded_address(switch_stun_packet_t *packet, char *ipstr, uint16_t port, int family);
+SWITCH_DECLARE(uint8_t) switch_stun_packet_attribute_add_integrity(switch_stun_packet_t *packet, const char *pass);
+SWITCH_DECLARE(uint32_t) switch_crc32_8bytes(const void* data, size_t length);
+SWITCH_DECLARE(uint8_t) switch_stun_packet_attribute_add_fingerprint(switch_stun_packet_t *packet);
+SWITCH_DECLARE(uint8_t) switch_stun_packet_attribute_add_use_candidate(switch_stun_packet_t *packet);
+SWITCH_DECLARE(uint8_t) switch_stun_packet_attribute_add_controlling(switch_stun_packet_t *packet);
+SWITCH_DECLARE(uint8_t) switch_stun_packet_attribute_add_controlled(switch_stun_packet_t *packet);
+SWITCH_DECLARE(uint8_t) switch_stun_packet_attribute_add_priority(switch_stun_packet_t *packet, uint32_t priority);
 
 /*!
   \brief Perform a stun lookup
@@ -209,13 +266,24 @@ SWITCH_DECLARE(uint8_t) switch_stun_packet_attribute_add_binded_address(switch_s
 SWITCH_DECLARE(switch_status_t) switch_stun_lookup(char **ip,
 												   switch_port_t *port, char *stunip, switch_port_t stunport, char **err, switch_memory_pool_t *pool);
 
+/*!
+  \brief Perform a stun ip lookup
+  \param external_ip replaced with stun results
+  \param sourceip stun:, host: or an ip
+  \param external_pool the memory pool to use
+  \return SUCCESS or FAIL
+*/
+SWITCH_DECLARE(switch_status_t) switch_stun_ip_lookup(char **external_ip, const char *sourceip, switch_memory_pool_t *external_pool);
+
 
 /*!
   \brief Obtain the padded length of an attribute's value
   \param attribute the attribute
   \return the padded size in bytes
 */
-#define switch_stun_attribute_padded_length(attribute) ((uint16_t)(attribute->length + (sizeof(uint32_t)-1)) & ~sizeof(uint32_t))
+#define switch_stun_attribute_padded_length(attribute) (int16_t)((attribute->length & 0x3) ? 0x4 + (attribute->length & ~0x3) : attribute->length)
+#define switch_stun_attribute_padded_length_hbo(attribute) (int16_t)((ntohs(attribute->length) & 0x3) ? 0x4 + (ntohs(attribute->length) & ~0x3) : ntohs(attribute->length))
+
 
 /*!
   \brief set a switch_stun_packet_attribute_t pointer to point at the first attribute in a packet
@@ -230,7 +298,10 @@ SWITCH_DECLARE(switch_status_t) switch_stun_lookup(char **ip,
   \param end pointer to the end of the buffer
   \return true or false depending on if there are any more attributes
 */
-#define switch_stun_packet_next_attribute(attribute, end) (attribute && (attribute = (switch_stun_packet_attribute_t *) (attribute->value +  switch_stun_attribute_padded_length(attribute))) && ((void *)attribute < end) && attribute->length && ((void *)(attribute +  switch_stun_attribute_padded_length(attribute)) < end))
+
+#define switch_stun_packet_next_attribute(attribute, end) (attribute && (attribute = (switch_stun_packet_attribute_t *) (attribute->value +  switch_stun_attribute_padded_length(attribute))) && ((void *)attribute < end) && attribute->type && (((switch_byte_t *)attribute +  switch_stun_attribute_padded_length(attribute)) < (switch_byte_t *)end))
+
+#define switch_stun_packet_next_attribute_hbo(attribute, end) (attribute && (attribute = (switch_stun_packet_attribute_t *) (attribute->value +  switch_stun_attribute_padded_length_hbo(attribute))) && ((void *)attribute < end) && attribute->type && (((switch_byte_t *)attribute +  switch_stun_attribute_padded_length_hbo(attribute)) < (switch_byte_t *)end))
 
 /*!
   \brief Obtain the correct length in bytes of a stun packet
@@ -250,5 +321,5 @@ SWITCH_END_EXTERN_C
  * c-basic-offset:4
  * End:
  * For VIM:
- * vim:set softtabstop=4 shiftwidth=4 tabstop=4:
+ * vim:set softtabstop=4 shiftwidth=4 tabstop=4 noet:
  */
