@@ -1,6 +1,6 @@
-/* 
+/*
  * FreeSWITCH Modular Media Switching Software Library / Soft-Switch Application
- * Copyright (C) 2005-2010, Anthony Minessale II <anthm@freeswitch.org>
+ * Copyright (C) 2005-2014, Anthony Minessale II <anthm@freeswitch.org>
  *
  * Version: MPL 1.1
  *
@@ -22,19 +22,21 @@
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
- * 
+ *
  * Anthony Minessale II <anthm@freeswitch.org>
+ * Jay Binks <jaybinks@gmail.com>
  *
  * mod_enum.c -- ENUM
  *
  */
 
 #include <switch.h>
-#include <udns.h>
-
-#ifndef WIN32
-#define closesocket close
+#ifdef _MSC_VER
+#define ssize_t int
 #endif
+#include <ldns/ldns.h>
+
+#define ENUM_MAXNAMESERVERS	10	/* max nameservers that will be used */
 
 SWITCH_MODULE_LOAD_FUNCTION(mod_enum_load);
 SWITCH_MODULE_SHUTDOWN_FUNCTION(mod_enum_shutdown);
@@ -49,19 +51,10 @@ struct enum_record {
 	char *route;
 	int supported;
 	struct enum_record *next;
+	struct enum_record *tail;
 };
 typedef struct enum_record enum_record_t;
 
-struct query {
-	const char *name;			/* original query string */
-	char *number;
-	unsigned char dn[DNS_MAXDN];
-	enum dns_type qtyp;			/* type of the query */
-	enum_record_t *results;
-	int errs;
-};
-typedef struct query enum_query_t;
-
 struct route {
 	char *service;
 	char *regex;
@@ -70,8 +63,6 @@ struct route {
 };
 typedef struct route enum_route_t;
 
-static enum dns_class qcls = DNS_C_IN;
-
 static switch_event_node_t *NODE = NULL;
 
 static struct {
@@ -81,6 +72,12 @@ static struct {
 	switch_memory_pool_t *pool;
 	int auto_reload;
 	int timeout;
+	int retries;
+	int random;
+	char *nameserver[ENUM_MAXNAMESERVERS];
+#ifdef _MSC_VER
+	char *nameserver_buf;
+#endif
 } globals;
 
 SWITCH_DECLARE_GLOBAL_STRING_FUNC(set_global_root, globals.root);
@@ -110,6 +107,7 @@ static void add_route(char *service, char *regex, char *replace)
 static switch_status_t load_config(void)
 {
 	char *cf = "enum.conf";
+	int inameserver = 0;
 	switch_xml_t cfg, xml = NULL, param, settings, route, routes;
 	switch_status_t status = SWITCH_STATUS_SUCCESS;
 
@@ -119,6 +117,10 @@ static switch_status_t load_config(void)
 		goto done;
 	}
 
+	globals.timeout = 5000;
+	globals.retries = 3;
+	globals.random  = 0;
+
 	if ((settings = switch_xml_child(cfg, "settings"))) {
 		for (param = switch_xml_child(settings, "param"); param; param = param->next) {
 			const char *var = switch_xml_attr_soft(param, "name");
@@ -128,11 +130,20 @@ static switch_status_t load_config(void)
 			} else if (!strcasecmp(var, "auto-reload")) {
 				globals.auto_reload = switch_true(val);
 			} else if (!strcasecmp(var, "query-timeout")) {
+				globals.timeout = atoi(val) * 1000;
+			} else if (!strcasecmp(var, "query-timeout-ms")) {
 				globals.timeout = atoi(val);
+			} else if (!strcasecmp(var, "query-timeout-retry")) {
+				globals.retries = atoi(val);
+			} else if (!strcasecmp(var, "random-nameserver")) {
+				globals.random = switch_true(val);
 			} else if (!strcasecmp(var, "default-isn-root")) {
 				set_global_isn_root(val);
-			} else if (!strcasecmp(var, "log-level-trace")) {
-
+			} else if (!strcasecmp(var, "nameserver") || !strcasecmp(var, "use-server")) {
+				if ( inameserver < ENUM_MAXNAMESERVERS ) {
+					globals.nameserver[inameserver] = (char *) val;
+					inameserver++;
+				}
 			}
 		}
 	}
@@ -152,6 +163,33 @@ static switch_status_t load_config(void)
 	}
 
   done:
+#ifdef _MSC_VER
+	if (!globals.nameserver[0]) {
+		HKEY hKey;
+		DWORD data_sz;
+		RegOpenKeyEx(HKEY_LOCAL_MACHINE,
+			"SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters",
+			0, KEY_QUERY_VALUE, &hKey);
+
+		if (hKey) {
+			RegQueryValueEx(hKey, "DhcpNameServer", NULL, NULL, NULL, &data_sz);
+			if (data_sz) {
+				globals.nameserver_buf = (char*)malloc(data_sz + 1);
+
+				RegQueryValueEx(hKey, "DhcpNameServer", NULL, NULL, (LPBYTE)globals.nameserver_buf, &data_sz);
+
+				if(globals.nameserver_buf[data_sz - 1] != 0) {
+					globals.nameserver_buf[data_sz] = 0;
+				}
+				switch_replace_char(globals.nameserver_buf, ' ', 0, SWITCH_FALSE); /* only use the first entry ex "192.168.1.1 192.168.1.2" */
+				globals.nameserver[0] = globals.nameserver_buf;
+			}
+			
+			RegCloseKey(hKey);
+		}
+	}
+#endif
+
 
 	if (xml) {
 		switch_xml_free(xml);
@@ -168,11 +206,12 @@ static switch_status_t load_config(void)
 	return status;
 }
 
-static char *reverse_number(char *in, char *root)
+static char *reverse_number(const char *in, const char *root)
 {
 	switch_size_t len;
 	char *out = NULL;
-	char *y, *z;
+	const char *y;
+	char *z;
 
 	if (!(in && root)) {
 		return NULL;
@@ -198,59 +237,33 @@ static char *reverse_number(char *in, char *root)
 	return out;
 }
 
-static void dnserror(enum_query_t *q, int errnum)
-{
 
-	switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_DEBUG, "Unable to lookup %s record for %s: %s\n",
-					  dns_typename(q->qtyp), dns_dntosp(q->dn), dns_strerror(errnum));
-	q->errs++;
-}
-
-static void add_result(enum_query_t *q, int order, int preference, char *service, char *route, int supported)
+static void add_result(enum_record_t **results, int order, int preference, char *service, char *route, int supported)
 {
-	enum_record_t *new_result, *rp, *prev = NULL;
+	enum_record_t *new_result;
 
 	new_result = malloc(sizeof(*new_result));
 	switch_assert(new_result);
-	memset(new_result, 0, sizeof(*new_result));
 
+	memset(new_result, 0, sizeof(*new_result));
 	new_result->order = order;
 	new_result->preference = preference;
 	new_result->service = strdup(service);
 	new_result->route = strdup(route);
 	new_result->supported = supported;
 
-	if (!q->results) {
-		q->results = new_result;
-		return;
-	}
 
-	rp = q->results;
-
-	while (rp && strcasecmp(rp->service, new_result->service)) {
-		prev = rp;
-		rp = rp->next;
-	}
-
-	while (rp && !strcasecmp(rp->service, new_result->service) && new_result->order > rp->order) {
-		prev = rp;
-		rp = rp->next;
-	}
-
-	while (rp && !strcasecmp(rp->service, new_result->service) && new_result->preference > rp->preference) {
-		prev = rp;
-		rp = rp->next;
-	}
-
-	if (prev) {
-		new_result->next = rp;
-		prev->next = new_result;
+	if (!*results) {
+		*results = new_result;
+		(*results)->tail = new_result;
 	} else {
-		new_result->next = rp;
-		q->results = new_result;
+		(*results)->tail->next = new_result;
+		(*results)->tail = new_result;
 	}
+
 }
 
+
 static void free_results(enum_record_t ** results)
 {
 	enum_record_t *fp, *rp;
@@ -265,201 +278,295 @@ static void free_results(enum_record_t ** results)
 	*results = NULL;
 }
 
-static void parse_rr(const struct dns_parse *p, enum_query_t *q, struct dns_rr *rr)
+
+static ldns_rdf *ldns_rdf_new_addr_frm_str(const char *str)
 {
-	const unsigned char *pkt = p->dnsp_pkt;
-	const unsigned char *end = p->dnsp_end;
-	const unsigned char *dptr = rr->dnsrr_dptr;
-	const unsigned char *dend = rr->dnsrr_dend;
-	unsigned char *dn = rr->dnsrr_dn;
-	const unsigned char *c;
-	char flags;
-	int order;
-	int preference;
+	ldns_rdf *a = NULL;
+
+	ldns_str2rdf_a(&a, str);
+
+	if (!a) {
+		/* maybe ip6 */
+		ldns_str2rdf_aaaa(&a, str);
+		if (!a) {
+			return NULL;
+		}
+	}
+	return a;
+}
+
+#define strip_quotes(_s) if (*_s == '"') _s++; if (end_of(_s) == '"') end_of(_s) = '\0'
+
+static void parse_naptr(const ldns_rr *naptr, const char *number, enum_record_t **results)
+{
+	char *str = ldns_rr2str(naptr);
+	char *argv[11] = { 0 };
+	int i, argc;
+	char *pack[4] = { 0 };
+
+	char *p;
+	int order = 10;
+	int preference = 100;
 	char *service = NULL;
-	char *regex = NULL;
-	char *replace = NULL;
-	char *ptr;
-	int argc = 0;
-	char *argv[4] = { 0 };
-	int n;
-	char string_arg[3][256] = { {0} };
+	char *packstr;
 
-	switch (rr->dnsrr_typ) {
-	case DNS_T_NAPTR:			/* prio weight port targetDN */
-		c = dptr;
-		c += 4;					/* order, pref */
+	char *regex, *replace;
 
-		for (n = 0; n < 3; ++n) {
-			if (c >= dend) {
-				goto xperr;
-			} else {
-				c += *c + 1;
-			}
+	if (zstr(str)) {
+		if (str != NULL) {
+			/* In this case ldns_rr2str returned a malloc'd null terminated string */
+			switch_safe_free(str);
 		}
+		return;
+	}
 
-		if (dns_getdn(pkt, &c, end, dn, DNS_MAXDN) <= 0 || c != dend) {
-			goto xperr;
+	for (p = str; p && *p; p++) {
+		if (*p == '\t') *p = ' ';
+		if (*p == ' ' && *(p+1) == '.') *p = '\0';
+	}
+
+
+	argc = switch_split(str, ' ', argv);
+
+	for (i = 0; i < argc; i++) {
+		if (i > 0) {
+			strip_quotes(argv[i]);
 		}
+	}
 
-		c = dptr;
-		order = dns_get16(c + 0);
-		preference = dns_get16(c + 2);
-		flags = (char) dns_get16(c + 4);
-		c += 4;
+	service = argv[7];
+	packstr = argv[8];
 
-		for (n = 0; n < 3; n++) {
-			uint32_t len = *c++, cpylen = len;
-			switch_assert(string_arg[n]);
-			if (len > sizeof(string_arg[n]) - 1) {
-				cpylen = sizeof(string_arg[n]) - 1;
-			}
-			strncpy(string_arg[n], (char *) c, cpylen);
-			*(string_arg[n] + len) = '\0';
-			c += len;
+	if (zstr(service) || zstr(packstr)) {
+		goto end;
+	}
+
+	if (!zstr(argv[4])) {
+		order = atoi(argv[4]);
+	}
+
+	if (!zstr(argv[5])) {
+		preference = atoi(argv[5]);
+	}
+
+
+	if (switch_split(packstr, '!', pack)) {
+		regex = pack[1];
+		replace = pack[2];
+	} else {
+		goto end;
+	}
+
+	for (p = replace; p && *p; p++) {
+		if (*p == '\\') {
+			*p = '$';
 		}
+	}
 
-		service = string_arg[1];
+	if (service && regex && replace) {
+		switch_regex_t *re = NULL, *re2 = NULL;
+		int proceed = 0, ovector[30];
+		char *substituted = NULL;
+		char *substituted_2 = NULL;
+		char *orig_uri;
+		char *uri_expanded = NULL;
+		enum_route_t *route;
+		int supported = 0;
+		uint32_t len = 0;
 
-		if ((argc = switch_separate_string(string_arg[2], '!', argv, (sizeof(argv) / sizeof(argv[0]))))) {
-			regex = argv[1];
-			replace = argv[2];
-		} else {
-			goto xperr;
-		}
-
-		for (ptr = replace; ptr && *ptr; ptr++) {
-			if (*ptr == '\\') {
-				*ptr = '$';
-			}
-		}
-
-		if (flags && service && regex && replace) {
-			switch_regex_t *re = NULL;
-			int proceed = 0, ovector[30];
-			char substituted[1024] = "";
-			char rbuf[1024] = "";
-			char *uri;
-			enum_route_t *route;
-			int supported = 0;
-			switch_regex_safe_free(re);
-
-			if ((proceed = switch_regex_perform(q->number, regex, &re, ovector, sizeof(ovector) / sizeof(ovector[0])))) {
-				if (strchr(regex, '(')) {
-					switch_perform_substitution(re, proceed, replace, q->number, substituted, sizeof(substituted), ovector);
-					uri = substituted;
-				} else {
-					uri = replace;
-				}
-
-				switch_mutex_lock(MUTEX);
-				for (route = globals.route_order; route; route = route->next) {
-					if (strcasecmp(service, route->service)) {
-						continue;
-					}
+		if ((proceed = switch_regex_perform(number, regex, &re, ovector, sizeof(ovector) / sizeof(ovector[0])))) {
+			if (strchr(regex, '(')) {
+				len = (uint32_t) (strlen(number) + strlen(replace) + 10) * proceed;
+				if (!(substituted = malloc(len))) {
+					switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_CRIT, "Memory Error!\n");
 					switch_regex_safe_free(re);
-					if ((proceed = switch_regex_perform(uri, route->regex, &re, ovector, sizeof(ovector) / sizeof(ovector[0])))) {
-						if (strchr(route->regex, '(')) {
-							switch_perform_substitution(re, proceed, route->replace, uri, rbuf, sizeof(rbuf), ovector);
-							uri = rbuf;
-						} else {
-							uri = route->replace;
-						}
-						supported++;
-						add_result(q, order, preference, service, uri, supported);
-					}
+					goto end;
+				}
+				memset(substituted, 0, len);
+
+				switch_perform_substitution(re, proceed, replace, number, substituted, len, ovector);
+				orig_uri = substituted;
+			} else {
+				orig_uri = replace;
+			}
+
+			switch_mutex_lock(MUTEX);
+			for (route = globals.route_order; route; route = route->next) {
+				char *uri = orig_uri;
+
+				if (strcasecmp(service, route->service)) {
+					continue;
 				}
 
-				if (!supported) {
-					add_result(q, order, preference, service, uri, 0);
+				if ((proceed = switch_regex_perform(uri, route->regex, &re2, ovector, sizeof(ovector) / sizeof(ovector[0])))) {
+					switch_event_t *event = NULL;
+
+					if (strchr(route->regex, '(')) {
+						len = (uint32_t) (strlen(uri) + strlen(route->replace) + 10) * proceed;
+						if (!(substituted_2 = malloc(len))) {
+							switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_CRIT, "Memory Error!\n");
+							switch_safe_free(substituted);
+							switch_regex_safe_free(re);
+							switch_regex_safe_free(re2);
+							switch_mutex_unlock(MUTEX);
+							goto end;
+						}
+						memset(substituted_2, 0, len);
+
+						switch_perform_substitution(re2, proceed, route->replace, uri, substituted_2, len, ovector);
+						uri = substituted_2;
+					} else {
+						uri = route->replace;
+					}
+					switch_event_create(&event, SWITCH_EVENT_REQUEST_PARAMS);
+					uri_expanded = switch_event_expand_headers(event, uri);
+					switch_event_destroy(&event);
+
+					if (uri_expanded == uri) {
+						uri_expanded = NULL;
+					} else {
+						uri = uri_expanded;
+					}
+
+					supported++;
+					add_result(results, order, preference, service, uri, supported);
+
 				}
+				switch_safe_free(uri_expanded);
+				switch_safe_free(substituted_2);
+				switch_regex_safe_free(re2);
 			}
 			switch_mutex_unlock(MUTEX);
 
+			if (!supported) {
+				add_result(results, order, preference, service, orig_uri, 0);
+			}
 
+			switch_safe_free(substituted);
 			switch_regex_safe_free(re);
 		}
-
-		break;
-
-	default:
-		break;
 	}
 
-	return;
+ end:
 
-  xperr:
+	switch_safe_free(str);
 
 	return;
 }
 
-static void dnscb(struct dns_ctx *ctx, void *result, void *data)
+switch_status_t ldns_lookup(const char *number, const char *root, char *server_name[ENUM_MAXNAMESERVERS] , enum_record_t **results)
 {
-	int r = dns_status(ctx);
-	enum_query_t *q = data;
-	struct dns_parse p;
-	struct dns_rr rr;
-	unsigned nrr;
-	unsigned char dn[DNS_MAXDN];
-	const unsigned char *pkt, *cur, *end, *qdn;
-	if (!result) {
-		dnserror(q, r);
-		return;
+	ldns_resolver *res = NULL;
+	ldns_rdf *domain = NULL;
+	ldns_pkt *p = NULL;
+	ldns_rr_list *naptr = NULL;
+	ldns_status s = LDNS_STATUS_ERR;
+	ldns_rdf *serv_rdf;
+	switch_status_t status = SWITCH_STATUS_FALSE;
+	char *name = NULL;
+	struct timeval to = { 0, 0};
+	int inameserver = 0;
+	int added_server = 0;
+
+	if (!(name = reverse_number(number, root))) {
+		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_DEBUG, "Parse Error!\n");
+		goto end;
 	}
-	pkt = result;
-	end = pkt + r;
-	cur = dns_payload(pkt);
-	dns_getdn(pkt, &cur, end, dn, sizeof(dn));
-	dns_initparse(&p, NULL, pkt, cur, end);
-	p.dnsp_qcls = 0;
-	p.dnsp_qtyp = 0;
-	qdn = dn;
-	nrr = 0;
-	while ((r = dns_nextrr(&p, &rr)) > 0) {
-		if (!dns_dnequal(qdn, rr.dnsrr_dn))
-			continue;
-		if ((qcls == DNS_C_ANY || qcls == rr.dnsrr_cls) && (q->qtyp == DNS_T_ANY || q->qtyp == rr.dnsrr_typ))
-			++nrr;
-		else if (rr.dnsrr_typ == DNS_T_CNAME && !nrr) {
-			if (dns_getdn(pkt, &rr.dnsrr_dptr, end, p.dnsp_dnbuf, sizeof(p.dnsp_dnbuf)) <= 0 || rr.dnsrr_dptr != rr.dnsrr_dend) {
-				r = DNS_E_PROTOCOL;
-				break;
-			} else {
-				qdn = p.dnsp_dnbuf;
+
+	if (!(domain = ldns_dname_new_frm_str(name))) {
+		goto end;
+	}
+
+	if (server_name) {
+		res = ldns_resolver_new();
+		switch_assert(res);
+
+		for(inameserver=0; inameserver<ENUM_MAXNAMESERVERS; inameserver++) {
+			if ( server_name[inameserver] != NULL ) {
+				switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_DEBUG, "Adding Nameserver [%s]\n", server_name[inameserver]);
+				if ((serv_rdf = ldns_rdf_new_addr_frm_str( server_name[inameserver] ))) {
+					s = ldns_resolver_push_nameserver(res, serv_rdf);
+					ldns_rdf_deep_free(serv_rdf);
+					added_server = 1;
+				}
 			}
 		}
 	}
-	if (!r && !nrr)
-		r = DNS_E_NODATA;
-	if (r < 0) {
-		dnserror(q, r);
-		free(result);
-		return;
+	if (!added_server) {
+		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_DEBUG, "No Nameservers specified, using host default\n");
+		/* create a new resolver from /etc/resolv.conf */
+		if (res) {
+			ldns_resolver_free(res);
+			res = NULL;
+		}
+		s = ldns_resolver_new_frm_file(&res, NULL);
 	}
 
-	dns_rewind(&p, NULL);
-	p.dnsp_qtyp = q->qtyp;
-	p.dnsp_qcls = qcls;
-	while (dns_nextrr(&p, &rr)) {
-		parse_rr(&p, q, &rr);
+	if (s != LDNS_STATUS_OK) {
+		goto end;
 	}
 
-	free(result);
+	to.tv_sec = globals.timeout / 1000;
+	to.tv_usec = (globals.timeout % 1000) * 1000;
+
+	ldns_resolver_set_timeout(res, to);
+	ldns_resolver_set_retry(res, (uint8_t)globals.retries);
+	ldns_resolver_set_random(res, globals.random);
+
+	if ((p = ldns_resolver_query(res,
+								 domain,
+								 LDNS_RR_TYPE_NAPTR,
+								 LDNS_RR_CLASS_IN,
+								 LDNS_RD))) {
+		/* retrieve the NAPTR records from the answer section of that
+		 * packet
+		 */
+
+		if ((naptr = ldns_pkt_rr_list_by_type(p, LDNS_RR_TYPE_NAPTR, LDNS_SECTION_ANSWER))) {
+			size_t i;
+
+			ldns_rr_list_sort(naptr);
+
+			for (i = 0; i < ldns_rr_list_rr_count(naptr); i++) {
+				parse_naptr(ldns_rr_list_rr(naptr, i), number, results);
+			}
+
+			//ldns_rr_list_print(stdout, naptr);
+			ldns_rr_list_deep_free(naptr);
+			status = SWITCH_STATUS_SUCCESS;
+		}
+	}
+
+ end:
+
+	switch_safe_free(name);
+
+	if (domain) {
+		ldns_rdf_deep_free(domain);
+	}
+
+	if (p) {
+		ldns_pkt_free(p);
+	}
+
+	if (res) {
+		ldns_resolver_deep_free(res);
+	}
+
+	return status;
 }
 
-static switch_status_t enum_lookup(char *root, char *in, enum_record_t ** results)
+static switch_status_t enum_lookup(char *root, char *in, enum_record_t **results, switch_channel_t *channel, switch_core_session_t *session)
 {
 	switch_status_t sstatus = SWITCH_STATUS_SUCCESS;
-	char *name = NULL;
-	enum_query_t query = { 0 };
-	enum dns_type l_qtyp = DNS_T_NAPTR;
-	int i = 0, abs = 0, j = 0;
-	dns_socket fd = (dns_socket) - 1;
-	fd_set fds;
-	struct timeval tv = { 0 };
-	time_t now = 0;
-	struct dns_ctx *nctx = NULL;
-	char *num, *mnum = NULL, *mroot = NULL, *p;
+	char *mnum = NULL, *mroot = NULL, *p;
+	char *server[ENUM_MAXNAMESERVERS];
+	int inameserver = 0;
+	char *argv[ ENUM_MAXNAMESERVERS ] = { 0 };
+	int argc;
+	int x = 0;
+	char *enum_nameserver_dup;
+	const char *enum_nameserver = NULL;
 
 	*results = NULL;
 
@@ -475,87 +582,52 @@ static switch_status_t enum_lookup(char *root, char *in, enum_record_t ** result
 		root = globals.root;
 	}
 
-	num = mnum;
-	if (!(name = reverse_number(num, root))) {
-		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_DEBUG, "Parse Error!\n");
-		sstatus = SWITCH_STATUS_FALSE;
-		goto done;
+	/* Empty the server array */
+	for(inameserver=0; inameserver<ENUM_MAXNAMESERVERS; inameserver++) {
+		server[inameserver] = NULL;
 	}
 
-	if (!(nctx = dns_new(NULL))) {
-		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_DEBUG, "Memory Error!\n");
-		sstatus = SWITCH_STATUS_MEMERR;
-		goto done;
+	inameserver = 0;
+
+	/* check for enum_nameserver channel var */
+
+	if (channel) {
+		enum_nameserver = switch_channel_get_variable(channel, "enum_nameserver");
 	}
 
-	fd = dns_open(nctx);
-
-	if (fd < 0) {
-		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_DEBUG, "FD Error!\n");
-		sstatus = SWITCH_STATUS_FALSE;
-		goto done;
+	if (zstr(enum_nameserver)) {
+		enum_nameserver = switch_core_get_variable("enum-server");
 	}
 
-	dns_ptodn(name, (unsigned int) strlen(name), query.dn, sizeof(query.dn), &abs);
-	query.name = name;
-	query.number = num;
-	query.qtyp = l_qtyp;
-
-	if (abs) {
-		abs = DNS_NOSRCH;
-	}
-
-	if (!dns_submit_dn(nctx, query.dn, qcls, l_qtyp, abs, 0, dnscb, &query)) {
-		dnserror(&query, dns_status(nctx));
-	}
-
-	FD_ZERO(&fds);
-	now = 0;
-
-	while ((i = dns_timeouts(nctx, 1, now)) > 0) {
-#ifdef _MSC_VER
-#pragma warning(push)
-#pragma warning(disable: 4389 4127)
-#endif
-		FD_SET(fd, &fds);
-#ifdef _MSC_VER
-#pragma warning(pop)
-#endif
-
-		j += i;
-
-		if (j > globals.timeout || query.results || query.errs) {
-			break;
+	if (!zstr(enum_nameserver)) {
+		/* Blank the server array */
+		for(inameserver=0; inameserver<ENUM_MAXNAMESERVERS; inameserver++) {
+			server[inameserver] = NULL;
 		}
 
-		tv.tv_sec = i;
-		tv.tv_usec = 0;
-		i = select((int) (fd + 1), &fds, 0, 0, &tv);
-		now = switch_epoch_time_now(NULL);
-		if (i > 0) {
-			dns_ioevent(nctx, now);
+		enum_nameserver_dup = switch_core_session_strdup(session, enum_nameserver);
+		argc = switch_separate_string(enum_nameserver_dup, ',', argv, (sizeof(argv) / sizeof(argv[0])));
+
+		inameserver = 0;
+		for (x = 0; x < argc; x++) {
+			server[inameserver] = argv[x];
+			inameserver++;
+		}
+		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_DEBUG, "Enum nameserver override : %s\n", enum_nameserver);
+	}
+
+	if (!inameserver) {
+		/* use config param "nameserver" ( can be up to ENUM_MAXNAMESERVERS ) */
+		for(inameserver = 0; inameserver<ENUM_MAXNAMESERVERS; inameserver++) {
+			server[inameserver] = NULL;
+			if ( globals.nameserver[inameserver] != NULL ) {
+				server[inameserver] = globals.nameserver[inameserver];
+			}
 		}
 	}
 
-	if (!query.results) {
-		sstatus = SWITCH_STATUS_FALSE;
-	}
+	ldns_lookup(mnum, root, server, results);
 
-	*results = query.results;
-	query.results = NULL;
-
-  done:
-
-	if (fd > -1) {
-		closesocket(fd);
-		fd = (dns_socket) - 1;
-	}
-
-	if (nctx) {
-		dns_free(nctx);
-	}
-
-	switch_safe_free(name);
 	switch_safe_free(mnum);
 	switch_safe_free(mroot);
 
@@ -575,7 +647,7 @@ SWITCH_STANDARD_DIALPLAN(enum_dialplan_hunt)
 
 	switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_DEBUG, "ENUM Lookup on %s\n", caller_profile->destination_number);
 
-	if (enum_lookup(dp, caller_profile->destination_number, &results) == SWITCH_STATUS_SUCCESS) {
+	if (enum_lookup(dp, caller_profile->destination_number, &results, channel, session) == SWITCH_STATUS_SUCCESS) {
 		if ((extension = switch_caller_extension_new(session, caller_profile->destination_number, caller_profile->destination_number)) == 0) {
 			switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_CRIT, "Memory Error!\n");
 			free_results(&results);
@@ -601,7 +673,6 @@ SWITCH_STANDARD_DIALPLAN(enum_dialplan_hunt)
 
 SWITCH_STANDARD_APP(enum_app_function)
 {
-	int argc = 0;
 	char *argv[4] = { 0 };
 	char *mydata = NULL;
 	char *dest = NULL, *root = NULL;
@@ -619,19 +690,21 @@ SWITCH_STANDARD_APP(enum_app_function)
 		return;
 	}
 
-	if ((argc = switch_separate_string(mydata, ' ', argv, (sizeof(argv) / sizeof(argv[0]))))) {
+	if (switch_separate_string(mydata, ' ', argv, (sizeof(argv) / sizeof(argv[0])))) {
 		dest = argv[0];
 		root = argv[1];
-		if (enum_lookup(root, dest, &results) == SWITCH_STATUS_SUCCESS) {
-			switch_event_header_t *hi;
-			if ((hi = switch_channel_variable_first(channel))) {
-				for (; hi; hi = hi->next) {
+		if (enum_lookup(root, dest, &results, channel, session) == SWITCH_STATUS_SUCCESS) {
+			switch_event_t *vars;
+
+			if (switch_channel_get_variables(channel, &vars) == SWITCH_STATUS_SUCCESS) {
+				switch_event_header_t *hi;
+				for (hi = vars->headers; hi; hi = hi->next) {
 					char *vvar = hi->name;
 					if (vvar && !strncmp(vvar, "enum_", 5)) {
 						switch_channel_set_variable(channel, (char *) vvar, NULL);
 					}
 				}
-				switch_channel_variable_last(channel);
+				switch_event_destroy(&vars);
 			}
 
 			for (rp = results; rp; rp = rp->next) {
@@ -663,7 +736,6 @@ SWITCH_STANDARD_APP(enum_app_function)
 
 SWITCH_STANDARD_API(enum_api)
 {
-	int argc = 0;
 	char *argv[4] = { 0 };
 	char *mydata = NULL;
 	char *dest = NULL, *root = NULL;
@@ -684,11 +756,11 @@ SWITCH_STANDARD_API(enum_api)
 		abort();
 	}
 
-	if ((argc = switch_separate_string(mydata, ' ', argv, (sizeof(argv) / sizeof(argv[0]))))) {
+	if (switch_separate_string(mydata, ' ', argv, (sizeof(argv) / sizeof(argv[0])))) {
 		dest = argv[0];
 		root = argv[1];
 		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_DEBUG, "Looking up %s@%s\n", dest, root);
-		if (enum_lookup(root, dest, &results) == SWITCH_STATUS_SUCCESS) {
+		if (enum_lookup(root, dest, &results, NULL, session) == SWITCH_STATUS_SUCCESS) {
 			for (rp = results; rp; rp = rp->next) {
 				if (!rp->supported) {
 					continue;
@@ -728,6 +800,8 @@ static void do_load(void)
 		switch_core_destroy_memory_pool(&globals.pool);
 	}
 
+	switch_safe_free(globals.root);
+	switch_safe_free(globals.isn_root);
 	memset(&globals, 0, sizeof(globals));
 	switch_core_new_memory_pool(&globals.pool);
 	globals.timeout = 10;
@@ -738,7 +812,6 @@ static void do_load(void)
 
 SWITCH_STANDARD_API(enum_function)
 {
-	int argc = 0;
 	char *argv[4] = { 0 };
 	enum_record_t *results, *rp;
 	char *mydata = NULL;
@@ -754,7 +827,7 @@ SWITCH_STANDARD_API(enum_function)
 		return SWITCH_STATUS_SUCCESS;
 	}
 
-	if ((argc = switch_separate_string(mydata, ' ', argv, (sizeof(argv) / sizeof(argv[0]))))) {
+	if (switch_separate_string(mydata, ' ', argv, (sizeof(argv) / sizeof(argv[0])))) {
 		dest = argv[0];
 		root = argv[1];
 		switch_assert(dest);
@@ -762,12 +835,14 @@ SWITCH_STANDARD_API(enum_function)
 		if (!strcasecmp(dest, "reload")) {
 			do_load();
 			stream->write_function(stream, "+OK ENUM Reloaded.\n");
+			switch_safe_free(mydata);
 			return SWITCH_STATUS_SUCCESS;
 
 		}
 
-		if (!enum_lookup(root, dest, &results) == SWITCH_STATUS_SUCCESS) {
+		if (enum_lookup(root, dest, &results, NULL, session) != SWITCH_STATUS_SUCCESS) {
 			stream->write_function(stream, "No Match!\n");
+			switch_safe_free(mydata);
 			return SWITCH_STATUS_SUCCESS;
 		}
 
@@ -795,7 +870,8 @@ SWITCH_STANDARD_API(enum_function)
 	} else {
 		stream->write_function(stream, "Invalid Input!\n");
 	}
-
+	switch_safe_free(mydata);
+	
 	return SWITCH_STATUS_SUCCESS;
 }
 
@@ -823,9 +899,6 @@ SWITCH_MODULE_LOAD_FUNCTION(mod_enum_load)
 		return SWITCH_STATUS_TERM;
 	}
 
-	if (dns_init(0) < 0) {
-		return SWITCH_STATUS_FALSE;
-	}
 
 	memset(&globals, 0, sizeof(globals));
 	do_load();
@@ -853,6 +926,9 @@ SWITCH_MODULE_SHUTDOWN_FUNCTION(mod_enum_shutdown)
 
 	switch_safe_free(globals.root);
 	switch_safe_free(globals.isn_root);
+#ifdef _MSC_VER
+	switch_safe_free(globals.nameserver_buf);
+#endif
 
 	return SWITCH_STATUS_UNLOAD;
 }
@@ -865,5 +941,5 @@ SWITCH_MODULE_SHUTDOWN_FUNCTION(mod_enum_shutdown)
  * c-basic-offset:4
  * End:
  * For VIM:
- * vim:set softtabstop=4 shiftwidth=4 tabstop=4:
+ * vim:set softtabstop=4 shiftwidth=4 tabstop=4 noet:
  */
