@@ -1,6 +1,6 @@
-/* 
+/*
  * FreeSWITCH Modular Media Switching Software Library / Soft-Switch Application
- * Copyright (C) 2005-2010, Anthony Minessale II <anthm@freeswitch.org>
+ * Copyright (C) 2005-2014, Anthony Minessale II <anthm@freeswitch.org>
  *
  * Version: MPL 1.1
  *
@@ -22,23 +22,24 @@
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
- * 
+ *
  * Brian West <brian@freeswitch.org>
  * Bret McDanel <trixter AT 0xdecafbad.com>
  * Justin Cassidy <xachenant@hotmail.com>
+ * Emmanuel Schmidbauer <eschmidbauer@gmail.com>
  *
  * mod_xml_cdr.c -- XML CDR Module to files or curl
  *
  */
-#include <sys/stat.h>
 #include <switch.h>
-#include <curl/curl.h>
+#include <sys/stat.h>
+#include <switch_curl.h>
 #define MAX_URLS 20
 
 #define ENCODING_NONE 0
 #define ENCODING_DEFAULT 1
 #define ENCODING_BASE64 2
-#define ENCODING_TEXTXML 2
+#define ENCODING_TEXTXML 3
 
 static struct {
 	char *cred;
@@ -46,6 +47,7 @@ static struct {
 	int url_count;
 	int url_index;
 	switch_thread_rwlock_t *log_path_lock;
+	switch_mutex_t *url_index_mutex;
 	char *base_log_dir;
 	char *base_err_log_dir;
 	char *log_dir;
@@ -66,15 +68,18 @@ static struct {
 	int prefix_a;
 	int disable100continue;
 	int rotate;
-	int auth_scheme;
+	long auth_scheme;
+	int timeout;
 	switch_memory_pool_t *pool;
+	switch_event_node_t *node;
+	char *cookie_file;
 } globals;
 
 SWITCH_MODULE_LOAD_FUNCTION(mod_xml_cdr_load);
 SWITCH_MODULE_SHUTDOWN_FUNCTION(mod_xml_cdr_shutdown);
 SWITCH_MODULE_DEFINITION(mod_xml_cdr, mod_xml_cdr_load, mod_xml_cdr_shutdown, NULL);
 
-/* this function would have access to the HTML returned by the webserver, we don't need it 
+/* this function would have access to the HTML returned by the webserver, we don't need it
  * and the default curl activity is to print to stdout, something not as desirable
  * so we have a dummy function here
  */
@@ -83,7 +88,7 @@ static size_t httpCallBack(char *buffer, size_t size, size_t nitems, void *outst
 	return size * nitems;
 }
 
-static switch_status_t set_xml_cdr_log_dirs()
+static switch_status_t set_xml_cdr_log_dirs(void)
 {
 	switch_time_exp_t tm;
 	char *path = NULL;
@@ -103,7 +108,7 @@ static switch_status_t set_xml_cdr_log_dirs()
 
 				dir_status = SWITCH_STATUS_SUCCESS;
 				if (switch_directory_exists(path, globals.pool) != SWITCH_STATUS_SUCCESS) {
-					dir_status = switch_dir_make(path, SWITCH_FPROT_OS_DEFAULT, globals.pool);
+					dir_status = switch_dir_make_recursive(path, SWITCH_DEFAULT_DIR_PERMS, globals.pool);
 				}
 
 				if (dir_status == SWITCH_STATUS_SUCCESS) {
@@ -125,6 +130,7 @@ static switch_status_t set_xml_cdr_log_dirs()
 			if ((path = switch_safe_strdup(globals.base_log_dir))) {
 				switch_thread_rwlock_wrlock(globals.log_path_lock);
 				switch_safe_free(globals.log_dir);
+				switch_dir_make_recursive(path, SWITCH_DEFAULT_DIR_PERMS, globals.pool);
 				globals.log_dir = path;
 				switch_thread_rwlock_unlock(globals.log_path_lock);
 			} else {
@@ -141,7 +147,7 @@ static switch_status_t set_xml_cdr_log_dirs()
 
 				dir_status = SWITCH_STATUS_SUCCESS;
 				if (switch_directory_exists(path, globals.pool) != SWITCH_STATUS_SUCCESS) {
-					dir_status = switch_dir_make(path, SWITCH_FPROT_OS_DEFAULT, globals.pool);
+					dir_status = switch_dir_make_recursive(path, SWITCH_DEFAULT_DIR_PERMS, globals.pool);
 				}
 
 				if (dir_status == SWITCH_STATUS_SUCCESS) {
@@ -163,6 +169,7 @@ static switch_status_t set_xml_cdr_log_dirs()
 			if ((path = switch_safe_strdup(globals.base_err_log_dir))) {
 				switch_thread_rwlock_wrlock(globals.log_path_lock);
 				switch_safe_free(globals.err_log_dir);
+				switch_dir_make_recursive(path, SWITCH_DEFAULT_DIR_PERMS, globals.pool);
 				globals.err_log_dir = path;
 				switch_thread_rwlock_unlock(globals.log_path_lock);
 			} else {
@@ -177,7 +184,7 @@ static switch_status_t set_xml_cdr_log_dirs()
 
 static switch_status_t my_on_reporting(switch_core_session_t *session)
 {
-	switch_xml_t cdr;
+	switch_xml_t cdr = NULL;
 	char *xml_text = NULL;
 	char *path = NULL;
 	char *curl_xml_text = NULL;
@@ -186,13 +193,16 @@ static switch_status_t my_on_reporting(switch_core_session_t *session)
 	int fd = -1;
 	uint32_t cur_try;
 	long httpRes;
-	CURL *curl_handle = NULL;
-	struct curl_slist *headers = NULL;
-	struct curl_slist *slist = NULL;
+	switch_CURL *curl_handle = NULL;
+	switch_curl_slist_t *headers = NULL;
+	switch_curl_slist_t *slist = NULL;
 	switch_channel_t *channel = switch_core_session_get_channel(session);
 	switch_status_t status = SWITCH_STATUS_FALSE;
 	int is_b;
 	const char *a_prefix = "";
+	char url_joiner = '?';
+	int prefix_a;
+	const char *prefix_a_var = NULL;
 
 	if (globals.shutdown) {
 		return SWITCH_STATUS_SUCCESS;
@@ -200,9 +210,19 @@ static switch_status_t my_on_reporting(switch_core_session_t *session)
 
 	is_b = channel && switch_channel_get_originator_caller_profile(channel);
 	if (!globals.log_b && is_b) {
-		return SWITCH_STATUS_SUCCESS;
+		const char *force_cdr = switch_channel_get_variable(channel, SWITCH_FORCE_PROCESS_CDR_VARIABLE);
+		if (!switch_true(force_cdr)) {
+			return SWITCH_STATUS_SUCCESS;
+		}
 	}
-	if (!is_b && globals.prefix_a)
+
+	// channel variable can over-ride global setting "prefix-a-leg"
+	if ((prefix_a_var = switch_channel_get_variable(channel, "prefix-a-leg"))) {
+		prefix_a = switch_true(prefix_a_var);
+	} else {
+		prefix_a = globals.prefix_a;
+	}
+	if (!is_b && prefix_a)
 		a_prefix = "a_";
 
 	if (switch_ivr_generate_xml_cdr(session, &cdr) != SWITCH_STATUS_SUCCESS) {
@@ -234,16 +254,12 @@ static switch_status_t my_on_reporting(switch_core_session_t *session)
 #endif
 				int wrote;
 				wrote = write(fd, xml_text, (unsigned) strlen(xml_text));
+				(void)wrote;
 				close(fd);
-				fd = -1;
 			} else {
 				char ebuf[512] = { 0 };
-#ifdef WIN32
-				strerror_s(ebuf, sizeof(ebuf), errno);
-#else
-				strerror_r(errno, ebuf, sizeof(ebuf));
-#endif
-				switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Error writing [%s][%s]\n", path, ebuf);
+				switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Error writing [%s][%s]\n",
+						path, switch_strerror_r(errno, ebuf, sizeof(ebuf)));
 			}
 			switch_safe_free(path);
 		}
@@ -254,27 +270,33 @@ static switch_status_t my_on_reporting(switch_core_session_t *session)
 	/* try to post it to the web server */
 	if (globals.url_count) {
 		char *destUrl = NULL;
-		curl_handle = curl_easy_init();
+		int g_url_index = -1;
+
+		switch_mutex_lock(globals.url_index_mutex);
+		g_url_index = globals.url_index;
+		switch_mutex_unlock(globals.url_index_mutex);
+
+		curl_handle = switch_curl_easy_init();
 
 		if (globals.encode == ENCODING_TEXTXML) {
-			headers = curl_slist_append(headers, "Content-Type: text/xml");
+			headers = switch_curl_slist_append(headers, "Content-Type: text/xml");
 		} else if (globals.encode) {
-			switch_size_t need_bytes = strlen(xml_text) * 3;
+			switch_size_t need_bytes = strlen(xml_text) * 3 + 1;
 
 			xml_text_escaped = malloc(need_bytes);
 			switch_assert(xml_text_escaped);
 			memset(xml_text_escaped, 0, need_bytes);
 			if (globals.encode == ENCODING_DEFAULT) {
-				headers = curl_slist_append(headers, "Content-Type: application/x-www-form-urlencoded");
-				switch_url_encode(xml_text, xml_text_escaped, need_bytes);
+				headers = switch_curl_slist_append(headers, "Content-Type: application/x-www-form-urlencoded");
+				switch_url_encode_opt(xml_text, xml_text_escaped, need_bytes, SWITCH_TRUE);
 			} else {
-				headers = curl_slist_append(headers, "Content-Type: application/x-www-form-base64-encoded");
+				headers = switch_curl_slist_append(headers, "Content-Type: application/x-www-form-base64-encoded");
 				switch_b64_encode((unsigned char *) xml_text, need_bytes / 3, (unsigned char *) xml_text_escaped, need_bytes);
 			}
 			switch_safe_free(xml_text);
 			xml_text = xml_text_escaped;
 		} else {
-			headers = curl_slist_append(headers, "Content-Type: application/x-www-form-plaintext");
+			headers = switch_curl_slist_append(headers, "Content-Type: application/x-www-form-plaintext");
 		}
 
 		if (globals.encode == ENCODING_TEXTXML) {
@@ -285,48 +307,55 @@ static switch_status_t my_on_reporting(switch_core_session_t *session)
 		}
 
 		if (!zstr(globals.cred)) {
-			curl_easy_setopt(curl_handle, CURLOPT_HTTPAUTH, globals.auth_scheme);
-			curl_easy_setopt(curl_handle, CURLOPT_USERPWD, globals.cred);
+			switch_curl_easy_setopt(curl_handle, CURLOPT_HTTPAUTH, globals.auth_scheme);
+			switch_curl_easy_setopt(curl_handle, CURLOPT_USERPWD, globals.cred);
 		}
 
-		curl_easy_setopt(curl_handle, CURLOPT_HTTPHEADER, headers);
-		curl_easy_setopt(curl_handle, CURLOPT_POST, 1);
-		curl_easy_setopt(curl_handle, CURLOPT_POSTFIELDS, curl_xml_text);
-		curl_easy_setopt(curl_handle, CURLOPT_USERAGENT, "freeswitch-xml/1.0");
-		curl_easy_setopt(curl_handle, CURLOPT_WRITEFUNCTION, httpCallBack);
+		switch_curl_easy_setopt(curl_handle, CURLOPT_HTTPHEADER, headers);
+		switch_curl_easy_setopt(curl_handle, CURLOPT_POST, 1);
+		switch_curl_easy_setopt(curl_handle, CURLOPT_NOSIGNAL, 1);
+		switch_curl_easy_setopt(curl_handle, CURLOPT_POSTFIELDS, curl_xml_text);
+		switch_curl_easy_setopt(curl_handle, CURLOPT_USERAGENT, "freeswitch-xml/1.0");
+		switch_curl_easy_setopt(curl_handle, CURLOPT_WRITEFUNCTION, httpCallBack);
 
 		if (globals.disable100continue) {
-			slist = curl_slist_append(slist, "Expect:");
-			curl_easy_setopt(curl_handle, CURLOPT_HTTPHEADER, slist);
+			slist = switch_curl_slist_append(slist, "Expect:");
+			switch_curl_easy_setopt(curl_handle, CURLOPT_HTTPHEADER, slist);
 		}
 
 		if (globals.ssl_cert_file) {
-			curl_easy_setopt(curl_handle, CURLOPT_SSLCERT, globals.ssl_cert_file);
+			switch_curl_easy_setopt(curl_handle, CURLOPT_SSLCERT, globals.ssl_cert_file);
 		}
 
 		if (globals.ssl_key_file) {
-			curl_easy_setopt(curl_handle, CURLOPT_SSLKEY, globals.ssl_key_file);
+			switch_curl_easy_setopt(curl_handle, CURLOPT_SSLKEY, globals.ssl_key_file);
 		}
 
 		if (globals.ssl_key_password) {
-			curl_easy_setopt(curl_handle, CURLOPT_SSLKEYPASSWD, globals.ssl_key_password);
+			switch_curl_easy_setopt(curl_handle, CURLOPT_SSLKEYPASSWD, globals.ssl_key_password);
 		}
 
 		if (globals.ssl_version) {
 			if (!strcasecmp(globals.ssl_version, "SSLv3")) {
-				curl_easy_setopt(curl_handle, CURLOPT_SSLVERSION, CURL_SSLVERSION_SSLv3);
+				switch_curl_easy_setopt(curl_handle, CURLOPT_SSLVERSION, CURL_SSLVERSION_SSLv3);
 			} else if (!strcasecmp(globals.ssl_version, "TLSv1")) {
-				curl_easy_setopt(curl_handle, CURLOPT_SSLVERSION, CURL_SSLVERSION_TLSv1);
+				switch_curl_easy_setopt(curl_handle, CURLOPT_SSLVERSION, CURL_SSLVERSION_TLSv1);
 			}
 		}
 
 		if (globals.ssl_cacert_file) {
-			curl_easy_setopt(curl_handle, CURLOPT_CAINFO, globals.ssl_cacert_file);
+			switch_curl_easy_setopt(curl_handle, CURLOPT_CAINFO, globals.ssl_cacert_file);
 		}
 
+		if (globals.cookie_file) {
+			switch_curl_easy_setopt(curl_handle, CURLOPT_COOKIEJAR, globals.cookie_file);
+			switch_curl_easy_setopt(curl_handle, CURLOPT_COOKIEFILE, globals.cookie_file);
+		}
+
+		switch_curl_easy_setopt(curl_handle, CURLOPT_TIMEOUT, globals.timeout);
+
 		/* these were used for testing, optionally they may be enabled if someone desires
-		   curl_easy_setopt(curl_handle, CURLOPT_TIMEOUT, 120); // tcp timeout
-		   curl_easy_setopt(curl_handle, CURLOPT_FOLLOWLOCATION, 1); // 302 recursion level
+		   switch_curl_easy_setopt(curl_handle, CURLOPT_FOLLOWLOCATION, 1); // 302 recursion level
 		 */
 
 		for (cur_try = 0; cur_try < globals.retries; cur_try++) {
@@ -334,41 +363,52 @@ static switch_status_t my_on_reporting(switch_core_session_t *session)
 				switch_yield(globals.delay * 1000000);
 			}
 
-			destUrl = switch_mprintf("%s?uuid=%s", globals.urls[globals.url_index], switch_core_session_get_uuid(session));
-			curl_easy_setopt(curl_handle, CURLOPT_URL, destUrl);
+			if( strchr(globals.urls[g_url_index], '?') != NULL ) {
+				url_joiner = '&';
+			}
+			destUrl = switch_mprintf("%s%cuuid=%s%s", globals.urls[g_url_index], url_joiner, a_prefix, switch_core_session_get_uuid(session));
+			switch_curl_easy_setopt(curl_handle, CURLOPT_URL, destUrl);
 
 			if (!strncasecmp(destUrl, "https", 5)) {
-				curl_easy_setopt(curl_handle, CURLOPT_SSL_VERIFYPEER, 0);
-				curl_easy_setopt(curl_handle, CURLOPT_SSL_VERIFYHOST, 0);
+				switch_curl_easy_setopt(curl_handle, CURLOPT_SSL_VERIFYPEER, 0);
+				switch_curl_easy_setopt(curl_handle, CURLOPT_SSL_VERIFYHOST, 0);
 			}
 
 			if (globals.enable_cacert_check) {
-				curl_easy_setopt(curl_handle, CURLOPT_SSL_VERIFYPEER, TRUE);
+				switch_curl_easy_setopt(curl_handle, CURLOPT_SSL_VERIFYPEER, TRUE);
 			}
 
 			if (globals.enable_ssl_verifyhost) {
-				curl_easy_setopt(curl_handle, CURLOPT_SSL_VERIFYHOST, 2);
+				switch_curl_easy_setopt(curl_handle, CURLOPT_SSL_VERIFYHOST, 2);
 			}
 
-			curl_easy_perform(curl_handle);
-			curl_easy_getinfo(curl_handle, CURLINFO_RESPONSE_CODE, &httpRes);
+			/* overrides default 300s timeout, could be usefull if the current web server is down to prevent long time waiting for nothing */
+			/* connection_timeout = retry_timeout  */
+			switch_curl_easy_setopt(curl_handle, CURLOPT_CONNECTTIMEOUT, !globals.delay ? 5 : (long)globals.delay);
+			switch_curl_easy_perform(curl_handle);
+			switch_curl_easy_getinfo(curl_handle, CURLINFO_RESPONSE_CODE, &httpRes);
 			switch_safe_free(destUrl);
-			if (httpRes == 200) {
+			if (httpRes >= 200 && httpRes <= 299) {
 				goto success;
 			} else {
 				switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Got error [%ld] posting to web server [%s]\n",
-								  httpRes, globals.urls[globals.url_index]);
-				globals.url_index++;
+								  httpRes, globals.urls[g_url_index]);
+				g_url_index++;
 				switch_assert(globals.url_count <= MAX_URLS);
-				if (globals.url_index >= globals.url_count) {
-					globals.url_index = 0;
+				if (g_url_index >= globals.url_count) {
+					g_url_index = 0;
 				}
-				switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Retry will be with url [%s]\n", globals.urls[globals.url_index]);
+				switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Retry will be with url [%s]\n", globals.urls[g_url_index]);
+				switch_mutex_lock(globals.url_index_mutex);
+				if (globals.url_index != g_url_index) {
+					globals.url_index = g_url_index;
+				}
+				switch_mutex_unlock(globals.url_index_mutex);
 			}
 		}
-		curl_easy_cleanup(curl_handle);
-		curl_slist_free_all(headers);
-		curl_slist_free_all(slist);
+		switch_curl_easy_cleanup(curl_handle);
+		switch_curl_slist_free_all(headers);
+		switch_curl_slist_free_all(slist);
 		slist = NULL;
 		headers = NULL;
 		curl_handle = NULL;
@@ -387,16 +427,12 @@ static switch_status_t my_on_reporting(switch_core_session_t *session)
 #endif
 				int wrote;
 				wrote = write(fd, xml_text, (unsigned) strlen(xml_text));
+				(void)wrote;
 				close(fd);
-				fd = -1;
 			} else {
 				char ebuf[512] = { 0 };
-#ifdef WIN32
-				strerror_s(ebuf, sizeof(ebuf), errno);
-#else
-				strerror_r(errno, ebuf, sizeof(ebuf));
-#endif
-				switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Error![%s]\n", ebuf);
+				switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Error![%s]\n",
+						switch_strerror_r(errno, ebuf, sizeof(ebuf)));
 			}
 		}
 	}
@@ -406,13 +442,13 @@ static switch_status_t my_on_reporting(switch_core_session_t *session)
 
   error:
 	if (curl_handle) {
-		curl_easy_cleanup(curl_handle);
+		switch_curl_easy_cleanup(curl_handle);
 	}
 	if (headers) {
-		curl_slist_free_all(headers);
+		switch_curl_slist_free_all(headers);
 	}
 	if (slist) {
-		curl_slist_free_all(slist);
+		switch_curl_slist_free_all(slist);
 	}
 	if (curl_xml_text != xml_text) {
 		switch_safe_free(curl_xml_text);
@@ -460,12 +496,14 @@ SWITCH_MODULE_LOAD_FUNCTION(mod_xml_cdr_load)
 
 	*module_interface = switch_loadable_module_create_module_interface(pool, modname);
 
-	if (switch_event_bind(modname, SWITCH_EVENT_TRAP, SWITCH_EVENT_SUBCLASS_ANY, event_handler, NULL) != SWITCH_STATUS_SUCCESS) {
+	memset(&globals, 0, sizeof(globals));
+
+	if (switch_event_bind_removable(modname, SWITCH_EVENT_TRAP, SWITCH_EVENT_SUBCLASS_ANY, event_handler, NULL, &globals.node) != SWITCH_STATUS_SUCCESS) {
 		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Couldn't bind!\n");
+		switch_core_remove_state_handler(&state_handlers);
 		return SWITCH_STATUS_GENERR;
 	}
 
-	memset(&globals, 0, sizeof(globals));
 	globals.log_http_and_disk = 0;
 	globals.log_b = 1;
 	globals.disable100continue = 0;
@@ -473,11 +511,15 @@ SWITCH_MODULE_LOAD_FUNCTION(mod_xml_cdr_load)
 	globals.auth_scheme = CURLAUTH_BASIC;
 
 	switch_thread_rwlock_create(&globals.log_path_lock, pool);
+	switch_mutex_init(&globals.url_index_mutex, SWITCH_MUTEX_NESTED, globals.pool);
 
 	/* parse the config */
 	if (!(xml = switch_xml_open_cfg(cf, &cfg, NULL))) {
 		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Open of %s failed\n", cf);
-		return SWITCH_STATUS_TERM;
+		switch_event_unbind(&globals.node);
+		switch_core_remove_state_handler(&state_handlers);
+		switch_thread_rwlock_destroy(globals.log_path_lock);
+		return SWITCH_STATUS_FALSE;
 	}
 
 	if ((settings = switch_xml_child(cfg, "settings"))) {
@@ -495,8 +537,16 @@ SWITCH_MODULE_LOAD_FUNCTION(mod_xml_cdr_load)
 				}
 			} else if (!strcasecmp(var, "log-http-and-disk")) {
 				globals.log_http_and_disk = switch_true(val);
+			} else if (!strcasecmp(var, "timeout")) {
+				int tmp = atoi(val);
+				if (tmp >= 0) {
+					globals.timeout = tmp;
+				} else {
+					globals.timeout = 0;
+					switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Can't set a negative timeout!\n");
+				}
 			} else if (!strcasecmp(var, "delay") && !zstr(val)) {
-				globals.delay = (uint32_t) atoi(val);
+				globals.delay = switch_atoui(val);
 			} else if (!strcasecmp(var, "log-b-leg")) {
 				globals.log_b = switch_true(val);
 			} else if (!strcasecmp(var, "prefix-a-leg")) {
@@ -512,41 +562,41 @@ SWITCH_MODULE_LOAD_FUNCTION(mod_xml_cdr_load)
 					globals.encode = switch_true(val) ? ENCODING_DEFAULT : ENCODING_NONE;
 				}
 			} else if (!strcasecmp(var, "retries") && !zstr(val)) {
-				globals.retries = (uint32_t) atoi(val);
+				globals.retries = switch_atoui(val);
 			} else if (!strcasecmp(var, "rotate") && !zstr(val)) {
 				globals.rotate = switch_true(val);
 			} else if (!strcasecmp(var, "log-dir")) {
 				if (zstr(val)) {
-					globals.base_log_dir = switch_mprintf("%s%sxml_cdr", SWITCH_GLOBAL_dirs.log_dir, SWITCH_PATH_SEPARATOR);
+					globals.base_log_dir = switch_core_sprintf(globals.pool, "%s%sxml_cdr", SWITCH_GLOBAL_dirs.log_dir, SWITCH_PATH_SEPARATOR);
 				} else {
 					if (switch_is_file_path(val)) {
 						globals.base_log_dir = switch_core_strdup(globals.pool, val);
 					} else {
-						globals.base_log_dir = switch_mprintf("%s%s%s", SWITCH_GLOBAL_dirs.log_dir, SWITCH_PATH_SEPARATOR, val);
+						globals.base_log_dir = switch_core_sprintf(globals.pool, "%s%s%s", SWITCH_GLOBAL_dirs.log_dir, SWITCH_PATH_SEPARATOR, val);
 					}
 				}
 			} else if (!strcasecmp(var, "err-log-dir")) {
 				if (zstr(val)) {
-					globals.base_err_log_dir = switch_mprintf("%s%sxml_cdr", SWITCH_GLOBAL_dirs.log_dir, SWITCH_PATH_SEPARATOR);
+					globals.base_err_log_dir = switch_core_sprintf(globals.pool, "%s%sxml_cdr", SWITCH_GLOBAL_dirs.log_dir, SWITCH_PATH_SEPARATOR);
 				} else {
 					if (switch_is_file_path(val)) {
 						globals.base_err_log_dir = switch_core_strdup(globals.pool, val);
 					} else {
-						globals.base_err_log_dir = switch_mprintf("%s%s%s", SWITCH_GLOBAL_dirs.log_dir, SWITCH_PATH_SEPARATOR, val);
+						globals.base_err_log_dir = switch_core_sprintf(globals.pool, "%s%s%s", SWITCH_GLOBAL_dirs.log_dir, SWITCH_PATH_SEPARATOR, val);
 					}
 				}
 			} else if (!strcasecmp(var, "enable-cacert-check") && switch_true(val)) {
 				globals.enable_cacert_check = 1;
 			} else if (!strcasecmp(var, "ssl-cert-path")) {
-				globals.ssl_cert_file = val;
+				globals.ssl_cert_file = switch_core_strdup(globals.pool, val);
 			} else if (!strcasecmp(var, "ssl-key-path")) {
-				globals.ssl_key_file = val;
+				globals.ssl_key_file = switch_core_strdup(globals.pool, val);
 			} else if (!strcasecmp(var, "ssl-key-password")) {
-				globals.ssl_key_password = val;
+				globals.ssl_key_password = switch_core_strdup(globals.pool, val);
 			} else if (!strcasecmp(var, "ssl-version")) {
-				globals.ssl_version = val;
+				globals.ssl_version = switch_core_strdup(globals.pool, val);
 			} else if (!strcasecmp(var, "ssl-cacert-file")) {
-				globals.ssl_cacert_file = val;
+				globals.ssl_cacert_file = switch_core_strdup(globals.pool, val);
 			} else if (!strcasecmp(var, "enable-ssl-verifyhost") && switch_true(val)) {
 				globals.enable_ssl_verifyhost = 1;
 			} else if (!strcasecmp(var, "auth-scheme")) {
@@ -564,27 +614,26 @@ SWITCH_MODULE_LOAD_FUNCTION(mod_xml_cdr_load)
 				} else if (!strcasecmp(val, "GSS-NEGOTIATE")) {
 					globals.auth_scheme |= CURLAUTH_GSSNEGOTIATE;
 				} else if (!strcasecmp(val, "any")) {
-					globals.auth_scheme = CURLAUTH_ANY;
-				}
-			}
-
-			if (zstr(globals.base_err_log_dir)) {
-				if (!zstr(globals.base_log_dir)) {
-					globals.base_err_log_dir = switch_core_strdup(globals.pool, globals.base_log_dir);
-				} else {
-					globals.base_err_log_dir = switch_core_sprintf(globals.pool, "%s%sxml_cdr", SWITCH_GLOBAL_dirs.log_dir, SWITCH_PATH_SEPARATOR);
+					globals.auth_scheme = (long)CURLAUTH_ANY;
 				}
+			} else if (!strcasecmp(var, "cookie-file")) {
+				globals.cookie_file = switch_core_strdup(globals.pool, val);
 			}
 		}
-	}
-	if (globals.retries < 0) {
-		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Retries is negative, setting to 0\n");
-		globals.retries = 0;
+
+		if (zstr(globals.base_err_log_dir)) {
+			if (!zstr(globals.base_log_dir)) {
+				globals.base_err_log_dir = switch_core_strdup(globals.pool, globals.base_log_dir);
+			} else {
+				globals.base_err_log_dir = switch_core_sprintf(globals.pool, "%s%sxml_cdr", SWITCH_GLOBAL_dirs.log_dir, SWITCH_PATH_SEPARATOR);
+			}
+		}
+
 	}
 
-	if (globals.retries && globals.delay <= 0) {
-		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Retries set but delay 0 setting to 5000ms\n");
-		globals.delay = 5000;
+	if (globals.retries && globals.delay == 0) {
+		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Retries set but delay 0 setting to 5 seconds\n");
+		globals.delay = 5;
 	}
 
 	globals.retries++;
@@ -592,6 +641,7 @@ SWITCH_MODULE_LOAD_FUNCTION(mod_xml_cdr_load)
 	set_xml_cdr_log_dirs();
 
 	switch_xml_free(xml);
+
 	return status;
 }
 
@@ -600,7 +650,14 @@ SWITCH_MODULE_SHUTDOWN_FUNCTION(mod_xml_cdr_shutdown)
 
 	globals.shutdown = 1;
 
+	switch_safe_free(globals.log_dir);
+	switch_safe_free(globals.err_log_dir);
+
+	switch_event_unbind(&globals.node);
 	switch_core_remove_state_handler(&state_handlers);
+
+	switch_thread_rwlock_destroy(globals.log_path_lock);
+
 	return SWITCH_STATUS_SUCCESS;
 }
 
@@ -612,5 +669,5 @@ SWITCH_MODULE_SHUTDOWN_FUNCTION(mod_xml_cdr_shutdown)
  * c-basic-offset:4
  * End:
  * For VIM:
- * vim:set softtabstop=4 shiftwidth=4 tabstop=4:
+ * vim:set softtabstop=4 shiftwidth=4 tabstop=4 noet:
  */
