@@ -1,6 +1,6 @@
-/* 
+/*
  * FreeSWITCH Modular Media Switching Software Library / Soft-Switch Application
- * Copyright (C) 2005-2010, Anthony Minessale II <anthm@freeswitch.org>
+ * Copyright (C) 2005-2014, Anthony Minessale II <anthm@freeswitch.org>
  *
  * Version: MPL 1.1
  *
@@ -22,7 +22,7 @@
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
- * 
+ *
  * Anthony Minessale II <anthm@freeswitch.org>
  * Bret McDanel <trixter AT 0xdecafbad.com>
  * Justin Cassidy <xachenant@hotmail.com>
@@ -31,7 +31,8 @@
  *
  */
 #include <switch.h>
-#include <curl/curl.h>
+#include <switch_curl.h>
+
 
 SWITCH_MODULE_LOAD_FUNCTION(mod_xml_curl_load);
 SWITCH_MODULE_SHUTDOWN_FUNCTION(mod_xml_curl_shutdown);
@@ -43,6 +44,7 @@ struct xml_binding {
 	char *url;
 	char *bindings;
 	char *cred;
+	char *bind_local;
 	int disable100continue;
 	int use_get_style;
 	uint32_t enable_cacert_check;
@@ -55,8 +57,9 @@ struct xml_binding {
 	char *cookie_file;
 	switch_hash_t *vars_map;
 	int use_dynamic_url;
-	int auth_scheme;
+	long auth_scheme;
 	int timeout;
+	switch_size_t curl_max_bytes;
 };
 
 static int keep_files_around = 0;
@@ -138,8 +141,10 @@ static size_t file_callback(void *ptr, size_t size, size_t nmemb, void *data)
 static switch_xml_t xml_url_fetch(const char *section, const char *tag_name, const char *key_name, const char *key_value, switch_event_t *params,
 								  void *user_data)
 {
+	switch_event_t *my_params = NULL;
 	char filename[512] = "";
-	CURL *curl_handle = NULL;
+	switch_CURL *curl_handle = NULL;
+	switch_CURLcode cc;
 	struct config_data config_data;
 	switch_xml_t xml = NULL;
 	char *data = NULL;
@@ -147,15 +152,15 @@ static switch_xml_t xml_url_fetch(const char *section, const char *tag_name, con
 	char uuid_str[SWITCH_UUID_FORMATTED_LENGTH + 1];
 	xml_binding_t *binding = (xml_binding_t *) user_data;
 	char *file_url;
-	struct curl_slist *slist = NULL;
+	switch_curl_slist_t *slist = NULL;
 	long httpRes = 0;
-	struct curl_slist *headers = NULL;
+	switch_curl_slist_t *headers = NULL;
 	char hostname[256] = "";
 	char basic_data[512];
 	char *uri = NULL;
 	char *dynamic_url = NULL;
 
-	gethostname(hostname, sizeof(hostname));
+    strncpy(hostname, switch_core_get_switchname(), sizeof(hostname) - 1);
 
 	if (!binding) {
 		return NULL;
@@ -181,6 +186,7 @@ static switch_xml_t xml_url_fetch(const char *section, const char *tag_name, con
 		if (!params) {
 			switch_event_create(&params, SWITCH_EVENT_REQUEST_PARAMS);
 			switch_assert(params);
+			my_params = params;
 		}
 
 		switch_event_add_header_string(params, SWITCH_STACK_TOP, "hostname", hostname);
@@ -203,102 +209,111 @@ static switch_xml_t xml_url_fetch(const char *section, const char *tag_name, con
 	switch_uuid_get(&uuid);
 	switch_uuid_format(uuid_str, &uuid);
 
-	switch_snprintf(filename, sizeof(filename), "%s%s.tmp.xml", SWITCH_GLOBAL_dirs.temp_dir, uuid_str);
-	curl_handle = curl_easy_init();
-	headers = curl_slist_append(headers, "Content-Type: application/x-www-form-urlencoded");
-
-	if (!strncasecmp(binding->url, "https", 5)) {
-		curl_easy_setopt(curl_handle, CURLOPT_SSL_VERIFYPEER, 0);
-		curl_easy_setopt(curl_handle, CURLOPT_SSL_VERIFYHOST, 0);
-	}
+	switch_snprintf(filename, sizeof(filename), "%s%s%s.tmp.xml", SWITCH_GLOBAL_dirs.temp_dir, SWITCH_PATH_SEPARATOR, uuid_str);
 
 	memset(&config_data, 0, sizeof(config_data));
 
 	config_data.name = filename;
-	config_data.max_bytes = XML_CURL_MAX_BYTES;
+	config_data.max_bytes = binding->curl_max_bytes;
 
 	if ((config_data.fd = open(filename, O_CREAT | O_RDWR | O_TRUNC, S_IRUSR | S_IWUSR)) > -1) {
-		if (!zstr(binding->cred)) {
-			curl_easy_setopt(curl_handle, CURLOPT_HTTPAUTH, binding->auth_scheme);
-			curl_easy_setopt(curl_handle, CURLOPT_USERPWD, binding->cred);
+		curl_handle = switch_curl_easy_init();
+		headers = switch_curl_slist_append(headers, "Content-Type: application/x-www-form-urlencoded");
+
+		if (!strncasecmp(binding->url, "https", 5)) {
+			switch_curl_easy_setopt(curl_handle, CURLOPT_SSL_VERIFYPEER, 0);
+			switch_curl_easy_setopt(curl_handle, CURLOPT_SSL_VERIFYHOST, 0);
 		}
-		curl_easy_setopt(curl_handle, CURLOPT_HTTPHEADER, headers);
+
+		if (!zstr(binding->cred)) {
+			switch_curl_easy_setopt(curl_handle, CURLOPT_HTTPAUTH, binding->auth_scheme);
+			switch_curl_easy_setopt(curl_handle, CURLOPT_USERPWD, binding->cred);
+		}
+		switch_curl_easy_setopt(curl_handle, CURLOPT_HTTPHEADER, headers);
 		if (binding->method != NULL)
-			curl_easy_setopt(curl_handle, CURLOPT_CUSTOMREQUEST, binding->method);
-		curl_easy_setopt(curl_handle, CURLOPT_POST, !binding->use_get_style);
-		curl_easy_setopt(curl_handle, CURLOPT_FOLLOWLOCATION, 1);
-		curl_easy_setopt(curl_handle, CURLOPT_MAXREDIRS, 10);
+			switch_curl_easy_setopt(curl_handle, CURLOPT_CUSTOMREQUEST, binding->method);
+		switch_curl_easy_setopt(curl_handle, CURLOPT_POST, !binding->use_get_style);
+		switch_curl_easy_setopt(curl_handle, CURLOPT_FOLLOWLOCATION, 1);
+		switch_curl_easy_setopt(curl_handle, CURLOPT_MAXREDIRS, 10);
 		if (!binding->use_get_style)
-			curl_easy_setopt(curl_handle, CURLOPT_POSTFIELDS, data);
-		curl_easy_setopt(curl_handle, CURLOPT_URL, binding->use_get_style ? uri : dynamic_url);
-		curl_easy_setopt(curl_handle, CURLOPT_WRITEFUNCTION, file_callback);
-		curl_easy_setopt(curl_handle, CURLOPT_WRITEDATA, (void *) &config_data);
-		curl_easy_setopt(curl_handle, CURLOPT_USERAGENT, "freeswitch-xml/1.0");
+			switch_curl_easy_setopt(curl_handle, CURLOPT_POSTFIELDS, data);
+		switch_curl_easy_setopt(curl_handle, CURLOPT_URL, binding->use_get_style ? uri : dynamic_url);
+		switch_curl_easy_setopt(curl_handle, CURLOPT_WRITEFUNCTION, file_callback);
+		switch_curl_easy_setopt(curl_handle, CURLOPT_WRITEDATA, (void *) &config_data);
+		switch_curl_easy_setopt(curl_handle, CURLOPT_USERAGENT, "freeswitch-xml/1.0");
+		switch_curl_easy_setopt(curl_handle, CURLOPT_NOSIGNAL, 1);
 
 		if (binding->timeout) {
-			curl_easy_setopt(curl_handle, CURLOPT_TIMEOUT, binding->timeout);
-			curl_easy_setopt(curl_handle, CURLOPT_NOSIGNAL, 1);
+			switch_curl_easy_setopt(curl_handle, CURLOPT_TIMEOUT, binding->timeout);
 		}
 
 		if (binding->disable100continue) {
-			slist = curl_slist_append(slist, "Expect:");
-			curl_easy_setopt(curl_handle, CURLOPT_HTTPHEADER, slist);
+			slist = switch_curl_slist_append(slist, "Expect:");
+			switch_curl_easy_setopt(curl_handle, CURLOPT_HTTPHEADER, slist);
 		}
 
 		if (binding->enable_cacert_check) {
-			curl_easy_setopt(curl_handle, CURLOPT_SSL_VERIFYPEER, TRUE);
+			switch_curl_easy_setopt(curl_handle, CURLOPT_SSL_VERIFYPEER, TRUE);
 		}
 
 		if (binding->ssl_cert_file) {
-			curl_easy_setopt(curl_handle, CURLOPT_SSLCERT, binding->ssl_cert_file);
+			switch_curl_easy_setopt(curl_handle, CURLOPT_SSLCERT, binding->ssl_cert_file);
 		}
 
 		if (binding->ssl_key_file) {
-			curl_easy_setopt(curl_handle, CURLOPT_SSLKEY, binding->ssl_key_file);
+			switch_curl_easy_setopt(curl_handle, CURLOPT_SSLKEY, binding->ssl_key_file);
 		}
 
 		if (binding->ssl_key_password) {
-			curl_easy_setopt(curl_handle, CURLOPT_SSLKEYPASSWD, binding->ssl_key_password);
+			switch_curl_easy_setopt(curl_handle, CURLOPT_SSLKEYPASSWD, binding->ssl_key_password);
 		}
 
 		if (binding->ssl_version) {
 			if (!strcasecmp(binding->ssl_version, "SSLv3")) {
-				curl_easy_setopt(curl_handle, CURLOPT_SSLVERSION, CURL_SSLVERSION_SSLv3);
+				switch_curl_easy_setopt(curl_handle, CURLOPT_SSLVERSION, CURL_SSLVERSION_SSLv3);
 			} else if (!strcasecmp(binding->ssl_version, "TLSv1")) {
-				curl_easy_setopt(curl_handle, CURLOPT_SSLVERSION, CURL_SSLVERSION_TLSv1);
+				switch_curl_easy_setopt(curl_handle, CURLOPT_SSLVERSION, CURL_SSLVERSION_TLSv1);
 			}
 		}
 
 		if (binding->ssl_cacert_file) {
-			curl_easy_setopt(curl_handle, CURLOPT_CAINFO, binding->ssl_cacert_file);
+			switch_curl_easy_setopt(curl_handle, CURLOPT_CAINFO, binding->ssl_cacert_file);
 		}
 
 		if (binding->enable_ssl_verifyhost) {
-			curl_easy_setopt(curl_handle, CURLOPT_SSL_VERIFYHOST, 2);
+			switch_curl_easy_setopt(curl_handle, CURLOPT_SSL_VERIFYHOST, 2);
 		}
 
 		if (binding->cookie_file) {
-			curl_easy_setopt(curl_handle, CURLOPT_COOKIEJAR, binding->cookie_file);
-			curl_easy_setopt(curl_handle, CURLOPT_COOKIEFILE, binding->cookie_file);
+			switch_curl_easy_setopt(curl_handle, CURLOPT_COOKIEJAR, binding->cookie_file);
+			switch_curl_easy_setopt(curl_handle, CURLOPT_COOKIEFILE, binding->cookie_file);
 		}
 
-		curl_easy_perform(curl_handle);
-		curl_easy_getinfo(curl_handle, CURLINFO_RESPONSE_CODE, &httpRes);
-		curl_easy_cleanup(curl_handle);
-		curl_slist_free_all(headers);
-		curl_slist_free_all(slist);
+		if (binding->bind_local) {
+			curl_easy_setopt(curl_handle, CURLOPT_INTERFACE, binding->bind_local);
+		}
+
+		cc = switch_curl_easy_perform(curl_handle);
+		if (cc && cc != CURLE_WRITE_ERROR) {
+			switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_WARNING, "CURL returned error:[%d] %s\n", cc, switch_curl_easy_strerror(cc));
+		}
+
+		switch_curl_easy_getinfo(curl_handle, CURLINFO_RESPONSE_CODE, &httpRes);
+		switch_curl_easy_cleanup(curl_handle);
+		switch_curl_slist_free_all(headers);
+		switch_curl_slist_free_all(slist);
 		close(config_data.fd);
 	} else {
 		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Error Opening temp file!\n");
 	}
 
 	if (config_data.err) {
-		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Error encountered!\n");
+		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Error encountered! [%s]\ndata: [%s]\n", binding->url, data);
 		xml = NULL;
 	} else {
 		if (httpRes == 200) {
 			if (!(xml = switch_xml_parse_file(filename))) {
-				switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Error Parsing Result!\n");
+				switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Error Parsing Result! [%s]\ndata: [%s]\n", binding->url, data);
 			}
 		} else {
 			switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Received HTTP error %ld trying to fetch %s\ndata: [%s]\n", httpRes, binding->url,
@@ -321,6 +336,11 @@ static switch_xml_t xml_url_fetch(const char *section, const char *tag_name, con
 		switch_safe_free(uri);
 	if (binding->use_dynamic_url && dynamic_url != binding->url)
 		switch_safe_free(dynamic_url);
+
+	if (my_params) {
+		switch_event_destroy(&my_params);
+	}
+
 	return xml;
 }
 
@@ -347,11 +367,13 @@ static switch_status_t do_config(void)
 	for (binding_tag = switch_xml_child(bindings_tag, "binding"); binding_tag; binding_tag = binding_tag->next) {
 		char *bname = (char *) switch_xml_attr_soft(binding_tag, "name");
 		char *url = NULL;
+		char *bind_local = NULL;
 		char *bind_cred = NULL;
 		char *bind_mask = NULL;
 		char *method = NULL;
 		int disable100continue = 1;
 		int use_dynamic_url = 0, timeout = 0;
+		switch_size_t curl_max_bytes = XML_CURL_MAX_BYTES;
 		uint32_t enable_cacert_check = 0;
 		char *ssl_cert_file = NULL;
 		char *ssl_key_file = NULL;
@@ -361,7 +383,7 @@ static switch_status_t do_config(void)
 		uint32_t enable_ssl_verifyhost = 0;
 		char *cookie_file = NULL;
 		hash_node_t *hash_node;
-		int auth_scheme = CURLAUTH_BASIC;
+		long auth_scheme = CURLAUTH_BASIC;
 		need_vars_map = 0;
 		vars_map = NULL;
 
@@ -392,7 +414,7 @@ static switch_status_t do_config(void)
 				} else if (!strcasecmp(val, "GSS-NEGOTIATE")) {
 					auth_scheme |= CURLAUTH_GSSNEGOTIATE;
 				} else if (!strcasecmp(val, "any")) {
-					auth_scheme = CURLAUTH_ANY;
+					auth_scheme = (long)CURLAUTH_ANY;
 				}
 			} else if (!strcasecmp(var, "disable-100-continue") && !switch_true(val)) {
 				disable100continue = 0;
@@ -425,7 +447,7 @@ static switch_status_t do_config(void)
 				use_dynamic_url = 1;
 			} else if (!strcasecmp(var, "enable-post-var")) {
 				if (!vars_map && need_vars_map == 0) {
-					if (switch_core_hash_init(&vars_map, globals.pool) != SWITCH_STATUS_SUCCESS) {
+					if (switch_core_hash_init(&vars_map) != SWITCH_STATUS_SUCCESS) {
 						need_vars_map = -1;
 						switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_WARNING, "Can't init params hash!\n");
 						continue;
@@ -433,10 +455,20 @@ static switch_status_t do_config(void)
 					need_vars_map = 1;
 				}
 
-				if (vars_map && val)
+				if (vars_map && val) {
 					if (switch_core_hash_insert(vars_map, val, ENABLE_PARAM_VALUE) != SWITCH_STATUS_SUCCESS) {
 						switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_WARNING, "Can't add %s to params hash!\n", val);
 					}
+				}
+			} else if (!strcasecmp(var, "bind-local")) {
+				bind_local = val;
+			} else if (!strcasecmp(var, "response-max-bytes")) {
+				int tmp = atoi(val);
+				if (tmp >= 0) {
+					curl_max_bytes = tmp;
+				} else {
+					switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Can't set a negative maximum response bytes!\n");
+				}
 			}
 		}
 
@@ -447,7 +479,7 @@ static switch_status_t do_config(void)
 			continue;
 		}
 
-		if (!(binding = malloc(sizeof(*binding)))) {
+		if (!(binding = switch_core_alloc(globals.pool, sizeof(*binding)))) {
 			if (vars_map)
 				switch_core_hash_destroy(&vars_map);
 			goto done;
@@ -456,20 +488,23 @@ static switch_status_t do_config(void)
 
 		binding->auth_scheme = auth_scheme;
 		binding->timeout = timeout;
-		binding->url = strdup(url);
+		binding->url = switch_core_strdup(globals.pool, url);
 		switch_assert(binding->url);
 
+		if (bind_local != NULL) {
+			binding->bind_local = switch_core_strdup(globals.pool, bind_local);
+		}
 		if (method != NULL) {
-			binding->method = strdup(method);
+			binding->method = switch_core_strdup(globals.pool, method);
 		} else {
 			binding->method = NULL;
 		}
 		if (bind_mask) {
-			binding->bindings = strdup(bind_mask);
+			binding->bindings = switch_core_strdup(globals.pool, bind_mask);
 		}
 
 		if (bind_cred) {
-			binding->cred = strdup(bind_cred);
+			binding->cred = switch_core_strdup(globals.pool, bind_cred);
 		}
 
 		binding->disable100continue = disable100continue;
@@ -478,29 +513,29 @@ static switch_status_t do_config(void)
 		binding->enable_cacert_check = enable_cacert_check;
 
 		if (ssl_cert_file) {
-			binding->ssl_cert_file = strdup(ssl_cert_file);
+			binding->ssl_cert_file = switch_core_strdup(globals.pool, ssl_cert_file);
 		}
 
 		if (ssl_key_file) {
-			binding->ssl_key_file = strdup(ssl_key_file);
+			binding->ssl_key_file = switch_core_strdup(globals.pool, ssl_key_file);
 		}
 
 		if (ssl_key_password) {
-			binding->ssl_key_password = strdup(ssl_key_password);
+			binding->ssl_key_password = switch_core_strdup(globals.pool, ssl_key_password);
 		}
 
 		if (ssl_version) {
-			binding->ssl_version = strdup(ssl_version);
+			binding->ssl_version = switch_core_strdup(globals.pool, ssl_version);
 		}
 
 		if (ssl_cacert_file) {
-			binding->ssl_cacert_file = strdup(ssl_cacert_file);
+			binding->ssl_cacert_file = switch_core_strdup(globals.pool, ssl_cacert_file);
 		}
 
 		binding->enable_ssl_verifyhost = enable_ssl_verifyhost;
 
 		if (cookie_file) {
-			binding->cookie_file = strdup(cookie_file);
+			binding->cookie_file = switch_core_strdup(globals.pool, cookie_file);
 		}
 
 		binding->vars_map = vars_map;
@@ -522,6 +557,8 @@ static switch_status_t do_config(void)
 
 		}
 
+		binding->curl_max_bytes = curl_max_bytes;
+
 		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_NOTICE, "Binding [%s] XML Fetch Function [%s] [%s]\n",
 						  zstr(bname) ? "N/A" : bname, binding->url, binding->bindings ? binding->bindings : "all");
 		switch_xml_bind_search_function(xml_url_fetch, switch_xml_parse_section_string(binding->bindings), binding);
@@ -547,9 +584,7 @@ SWITCH_MODULE_LOAD_FUNCTION(mod_xml_curl_load)
 	globals.hash_root = NULL;
 	globals.hash_tail = NULL;
 
-	if (do_config() == SWITCH_STATUS_SUCCESS) {
-		curl_global_init(CURL_GLOBAL_ALL);
-	} else {
+	if (do_config() != SWITCH_STATUS_SUCCESS) {
 		return SWITCH_STATUS_FALSE;
 	}
 
@@ -573,7 +608,7 @@ SWITCH_MODULE_SHUTDOWN_FUNCTION(mod_xml_curl_shutdown)
 	}
 
 	switch_xml_unbind_search_function_ptr(xml_url_fetch);
-	curl_global_cleanup();
+
 	return SWITCH_STATUS_SUCCESS;
 }
 
@@ -585,5 +620,5 @@ SWITCH_MODULE_SHUTDOWN_FUNCTION(mod_xml_curl_shutdown)
  * c-basic-offset:4
  * End:
  * For VIM:
- * vim:set softtabstop=4 shiftwidth=4 tabstop=4:
+ * vim:set softtabstop=4 shiftwidth=4 tabstop=4 noet:
  */
