@@ -1,6 +1,6 @@
-/* 
+/*
  * FreeSWITCH Modular Media Switching Software Library / Soft-Switch Application
- * Copyright (C) 2005-2010, Anthony Minessale II <anthm@freeswitch.org>
+ * Copyright (C) 2005-2014, Anthony Minessale II <anthm@freeswitch.org>
  *
  * Version: MPL 1.1
  *
@@ -22,7 +22,7 @@
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
- * 
+ *
  * Anthony Minessale II <anthm@freeswitch.org>
  *
  *
@@ -32,10 +32,12 @@
 
 #include <switch.h>
 #include <switch_console.h>
-#include <switch_version.h>
+#ifndef _MSC_VER
+#include <switch_private.h>
+#endif
 #define CMD_BUFLEN 1024
 
-#ifdef SWITCH_HAVE_LIBEDIT
+#ifdef HAVE_LIBEDIT
 #include <histedit.h>
 
 static EditLine *el;
@@ -67,6 +69,7 @@ static char *hfile = NULL;
 #define KEY_RIGHT 7
 #define KEY_INSERT 8
 #define PROMPT_OP 9
+#define KEY_DELETE 10
 
 static int console_bufferInput(char *buf, int len, char *cmd, int key);
 #endif
@@ -105,7 +108,7 @@ static switch_status_t console_xml_config(void)
 		for (param = switch_xml_child(settings, "key"); param; param = param->next) {
 			char *var = (char *) switch_xml_attr_soft(param, "name");
 			char *val = (char *) switch_xml_attr_soft(param, "value");
-			int i = atoi(var);
+			i = atoi(var);
 			if ((i < 1) || (i > 12)) {
 				switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Keybind %s is invalid, range is from 1 to 12\n", var);
 			} else {
@@ -147,7 +150,6 @@ SWITCH_DECLARE_NONSTD(switch_status_t) switch_console_stream_raw_write(switch_st
 SWITCH_DECLARE_NONSTD(switch_status_t) switch_console_stream_write(switch_stream_handle_t *handle, const char *fmt, ...)
 {
 	va_list ap;
-	char *buf = handle->data;
 	char *end = handle->end;
 	int ret = 0;
 	char *data = NULL;
@@ -175,7 +177,6 @@ SWITCH_DECLARE_NONSTD(switch_status_t) switch_console_stream_write(switch_stream
 			if ((new_data = realloc(handle->data, new_len))) {
 				handle->data_size = handle->alloc_len = new_len;
 				handle->data = new_data;
-				buf = handle->data;
 				remaining = handle->data_size - handle->data_len;
 				handle->end = (uint8_t *) (handle->data) + handle->data_len;
 				end = handle->end;
@@ -191,7 +192,7 @@ SWITCH_DECLARE_NONSTD(switch_status_t) switch_console_stream_write(switch_stream
 		} else {
 			ret = 0;
 			switch_snprintf(end, remaining, "%s", data);
-			handle->data_len = strlen(buf);
+			handle->data_len += strlen(data);
 			handle->end = (uint8_t *) (handle->data) + handle->data_len;
 		}
 		free(data);
@@ -203,7 +204,7 @@ SWITCH_DECLARE_NONSTD(switch_status_t) switch_console_stream_write(switch_stream
 SWITCH_DECLARE(switch_status_t) switch_stream_write_file_contents(switch_stream_handle_t *stream, const char *path)
 {
 	char *dpath = NULL;
-	int fd;
+	FILE *fd = NULL;
 	switch_status_t status = SWITCH_STATUS_FALSE;
 
 	if (!switch_is_file_path(path)) {
@@ -211,12 +212,15 @@ SWITCH_DECLARE(switch_status_t) switch_stream_write_file_contents(switch_stream_
 		path = dpath;
 	}
 
-	if ((fd = open(path, O_RDONLY)) > -1) {
-		char buf[2048] = { 0 };
-		while (switch_fd_read_line(fd, buf, sizeof(buf))) {
-			stream->write_function(stream, "%s", buf);
+	if ((fd = fopen(path, "r"))) {
+		char *line_buf = NULL;
+		switch_size_t llen = 0;
+
+		while (switch_fp_read_dline(fd, &line_buf, &llen)) {
+			stream->write_function(stream, "%s", line_buf);
 		}
-		close(fd);
+		fclose(fd);
+		switch_safe_free(line_buf);
 		status = SWITCH_STATUS_SUCCESS;
 	}
 
@@ -238,16 +242,21 @@ SWITCH_DECLARE(char *) switch_console_expand_alias(char *cmd, char *arg)
 	char *sql = NULL;
 	char *exp = NULL;
 	switch_cache_db_handle_t *db = NULL;
+	switch_core_flag_t cflags = switch_core_flags();
 	int full = 0;
 
-	
+
+	if (!(cflags & SCF_USE_SQL)) {
+		return NULL;
+	}
+
 	if (switch_core_db_handle(&db) != SWITCH_STATUS_SUCCESS) {
 		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Database Error\n");
 		return NULL;
 	}
 
 
-	if (db->type == SCDB_TYPE_CORE_DB) {
+	if (switch_cache_db_get_type(db) == SCDB_TYPE_CORE_DB) {
 		sql = switch_mprintf("select command from aliases where alias='%q'", cmd);
 	} else {
 		sql = switch_mprintf("select command from aliases where alias='%w'", cmd);
@@ -263,7 +272,7 @@ SWITCH_DECLARE(char *) switch_console_expand_alias(char *cmd, char *arg)
 	switch_safe_free(sql);
 
 	if (!r) {
-		if (db->type == SCDB_TYPE_CORE_DB) {
+		if (switch_cache_db_get_type(db) == SCDB_TYPE_CORE_DB) {
 			sql = switch_mprintf("select command from aliases where alias='%q %q'", cmd, arg);
 		} else {
 			sql = switch_mprintf("select command from aliases where alias='%w %w'", cmd, arg);
@@ -341,7 +350,7 @@ SWITCH_DECLARE(switch_status_t) switch_console_execute(char *xcmd, int rec, swit
 	int argc;
 	char *argv[128];
 	int x;
-	char *dup = strdup(xcmd);
+	char *dup = NULL;
 	char *cmd;
 
 	switch_status_t status = SWITCH_STATUS_FALSE;
@@ -352,6 +361,10 @@ SWITCH_DECLARE(switch_status_t) switch_console_execute(char *xcmd, int rec, swit
 		goto end;
 	}
 
+	while (*xcmd == ' ') xcmd++;
+
+	dup = strdup(xcmd);
+
 	if (!strncasecmp(xcmd, "alias", 5)) {
 		argc = 1;
 		argv[0] = xcmd;
@@ -438,11 +451,8 @@ SWITCH_DECLARE(void) switch_console_printf(switch_text_channel_t channel, const
 	fflush(handle);
 }
 
-static char hostname[256] = "";
 static int32_t running = 1;
 
-
-
 struct helper {
 	int len;
 	int hits;
@@ -483,9 +493,29 @@ static int comp_callback(void *pArg, int argc, char **argv, char **columnNames)
 		return -1;
 	}
 
+	if (!zstr(target) && *target == ':' && *(target + 1) == ':' && *(target + 2) == '[') {
+		char *p = target + 3, *list = NULL;
+
+		if (p) {
+			char *s_argv[100] = { 0 };
+			char *r_argv[1] = { 0 }, *r_cols[1] = {0};
+			list = strdup(p);
+
+			argc = switch_separate_string(list, ':', s_argv, (sizeof(s_argv) / sizeof(s_argv[0])));
+
+			for (i = 0; (int)i < argc; i++) {
+				if (!cur || !strncmp(s_argv[i], cur, strlen(cur))) {
+					r_argv[0] = s_argv[i];
+					comp_callback(h, 1, r_argv, r_cols);
+				}
+			}
+			switch_safe_free(list);
+		}
+		return 0;
+	}
+
 	if (!zstr(target) && *target == ':' && *(target + 1) == ':') {
-		char *r_argv[1] = { 0 }, *r_cols[1] = {
-		0};
+		char *r_argv[1] = { 0 }, *r_cols[1] = {0};
 		switch_console_callback_match_t *matches;
 		if (switch_console_run_complete_func(target, str, cur, &matches) == SWITCH_STATUS_SUCCESS) {
 			switch_console_callback_match_node_t *m;
@@ -553,6 +583,76 @@ struct match_helper {
 	switch_console_callback_match_t *my_matches;
 };
 
+static int modulename_callback(void *pArg, const char *module_name)
+{
+	struct match_helper *h = (struct match_helper *) pArg;
+
+	switch_console_push_match(&h->my_matches, module_name);
+	return 0;
+}
+
+SWITCH_DECLARE_NONSTD(switch_status_t) switch_console_list_available_modules(const char *line, const char *cursor, switch_console_callback_match_t **matches)
+{
+	struct match_helper h = { 0 };
+
+	if (switch_loadable_module_enumerate_available(SWITCH_GLOBAL_dirs.mod_dir, modulename_callback, &h) != SWITCH_STATUS_SUCCESS) {
+		return SWITCH_STATUS_GENERR;
+	}
+
+	if (h.my_matches) {
+		*matches = h.my_matches;
+		return SWITCH_STATUS_SUCCESS;
+	}
+
+	return SWITCH_STATUS_FALSE;
+}
+
+SWITCH_DECLARE_NONSTD(switch_status_t) switch_console_list_loaded_modules(const char *line, const char *cursor, switch_console_callback_match_t **matches)
+{
+	struct match_helper h = { 0 };
+
+	if (switch_loadable_module_enumerate_loaded(modulename_callback, &h) != SWITCH_STATUS_SUCCESS) {
+		return SWITCH_STATUS_GENERR;
+	}
+
+	if (h.my_matches) {
+		*matches = h.my_matches;
+		return SWITCH_STATUS_SUCCESS;
+	}
+
+	return SWITCH_STATUS_FALSE;
+}
+
+#ifdef HAVE_GETIFADDRS
+#include <ifaddrs.h>
+#include <net/if.h>
+SWITCH_DECLARE_NONSTD(switch_status_t) switch_console_list_interfaces(const char *line, const char *cursor, switch_console_callback_match_t **matches)
+{
+	struct match_helper h = { 0 };
+	struct ifaddrs *addrs, *addr;
+
+	getifaddrs(&addrs);
+	for(addr = addrs; addr; addr = addr->ifa_next) {
+		if (addr->ifa_flags & IFF_UP) {
+			switch_console_push_match_unique(&h.my_matches, addr->ifa_name);
+		}
+	}
+	freeifaddrs(addrs);
+
+	if (h.my_matches) {
+		*matches = h.my_matches;
+		return SWITCH_STATUS_SUCCESS;
+	}
+
+	return SWITCH_STATUS_FALSE;
+}
+#else
+SWITCH_DECLARE_NONSTD(switch_status_t) switch_console_list_interfaces(const char *line, const char *cursor, switch_console_callback_match_t **matches)
+{
+	return SWITCH_STATUS_FALSE;
+}
+#endif
+
 static int uuid_callback(void *pArg, int argc, char **argv, char **columnNames)
 {
 	struct match_helper *h = (struct match_helper *) pArg;
@@ -562,7 +662,7 @@ static int uuid_callback(void *pArg, int argc, char **argv, char **columnNames)
 
 }
 
-SWITCH_DECLARE(switch_status_t) switch_console_list_uuid(const char *line, const char *cursor, switch_console_callback_match_t **matches)
+SWITCH_DECLARE_NONSTD(switch_status_t) switch_console_list_uuid(const char *line, const char *cursor, switch_console_callback_match_t **matches)
 {
 	char *sql;
 	struct match_helper h = { 0 };
@@ -578,9 +678,9 @@ SWITCH_DECLARE(switch_status_t) switch_console_list_uuid(const char *line, const
 
 	if (!zstr(cursor)) {
 		sql = switch_mprintf("select distinct uuid from channels where uuid like '%q%%' and hostname='%q' order by uuid",
-							 cursor, switch_core_get_variable("hostname"));
+							 cursor, switch_core_get_switchname());
 	} else {
-		sql = switch_mprintf("select distinct uuid from channels where hostname='%q' order by uuid", switch_core_get_variable("hostname"));
+		sql = switch_mprintf("select distinct uuid from channels where hostname='%q' order by uuid", switch_core_get_switchname());
 	}
 
 	switch_cache_db_execute_sql_callback(db, sql, uuid_callback, &h, &errmsg);
@@ -612,7 +712,7 @@ SWITCH_DECLARE(unsigned char) switch_console_complete(const char *line, const ch
 	int pos = 0;
 	int sc = 0;
 
-#ifndef SWITCH_HAVE_LIBEDIT
+#ifndef HAVE_LIBEDIT
 #ifndef _MSC_VER
 	if (!stream) {
 		return CC_ERROR;
@@ -622,11 +722,12 @@ SWITCH_DECLARE(unsigned char) switch_console_complete(const char *line, const ch
 
 	if (switch_core_db_handle(&db) != SWITCH_STATUS_SUCCESS) {
 		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Database Error\n");
+		switch_safe_free(dup);
 		return CC_ERROR;
 	}
 
 	if (!zstr(cursor) && !zstr(line)) {
-		pos = (cursor - line);
+		pos = (int)(cursor - line);
 	}
 
 	h.out = console_out;
@@ -647,7 +748,7 @@ SWITCH_DECLARE(unsigned char) switch_console_complete(const char *line, const ch
 	}
 
 	if (!*buf) {
-#ifdef SWITCH_HAVE_LIBEDIT
+#ifdef HAVE_LIBEDIT
 		if (h.out && sc) {
 			el_deletestr(el, sc);
 		}
@@ -662,7 +763,7 @@ SWITCH_DECLARE(unsigned char) switch_console_complete(const char *line, const ch
 	}
 
 	if (sc > 1) {
-#ifdef SWITCH_HAVE_LIBEDIT
+#ifdef HAVE_LIBEDIT
 		if (h.out) {
 			el_deletestr(el, sc - 1);
 		}
@@ -685,7 +786,7 @@ SWITCH_DECLARE(unsigned char) switch_console_complete(const char *line, const ch
 		buf = lp + 1;
 	}
 
-	h.len = strlen(buf);
+	h.len = (int)strlen(buf);
 
 	if (h.out) {
 		fprintf(h.out, "\n\n");
@@ -699,7 +800,7 @@ SWITCH_DECLARE(unsigned char) switch_console_complete(const char *line, const ch
 
 	if (h.words == 0) {
 		sql = switch_mprintf("select distinct name from interfaces where type='api' and name like '%q%%' and hostname='%q' order by name",
-							 buf, switch_core_get_variable("hostname"));
+							 buf, switch_core_get_hostname());
 	}
 
 	if (sql) {
@@ -726,19 +827,19 @@ SWITCH_DECLARE(unsigned char) switch_console_complete(const char *line, const ch
 		argc = switch_separate_string(dupdup, ' ', argv, (sizeof(argv) / sizeof(argv[0])));
 
 		if (h.words == 0) {
-			stream.write_function(&stream, "select distinct a1 from complete where " "a1 not in (select name from interfaces where hostname='%s') %s ",
-								  switch_core_get_variable("hostname"), argc ? "and" : "");
+			stream.write_function(&stream, "select distinct a1 from complete where " "a1 not in (select name from interfaces where hostname='%q') %s ",
+								  switch_core_get_hostname(), argc ? "and" : "");
 		} else {
-			if (db->type == SCDB_TYPE_CORE_DB) {
+			if (switch_cache_db_get_type(db) == SCDB_TYPE_CORE_DB) {
 				stream.write_function(&stream, "select distinct a%d,'%q','%q' from complete where ", h.words + 1, switch_str_nil(dup), switch_str_nil(lp));
 			} else {
 				stream.write_function(&stream, "select distinct a%d,'%q','%w' from complete where ", h.words + 1, switch_str_nil(dup), switch_str_nil(lp));
 			}
 		}
 
-		for (x = 0; x < argc && x < 11; x++) {
+		for (x = 0; x < argc; x++) {
 			if (h.words + 1 > argc) {
-				if (db->type == SCDB_TYPE_CORE_DB) {
+				if (switch_cache_db_get_type(db) == SCDB_TYPE_CORE_DB) {
 					stream.write_function(&stream, "(a%d like '::%%' or a%d = '' or a%d = '%q')%q",
 										  x + 1, x + 1, x + 1, switch_str_nil(argv[x]), x == argc - 1 ? "" : " and ");
 				} else {
@@ -746,7 +847,7 @@ SWITCH_DECLARE(unsigned char) switch_console_complete(const char *line, const ch
 										  x + 1, x + 1, x + 1, switch_str_nil(argv[x]), x == argc - 1 ? "" : " and ");
 				}
 			} else {
-				if (db->type == SCDB_TYPE_CORE_DB) {
+				if (switch_cache_db_get_type(db) == SCDB_TYPE_CORE_DB) {
 					stream.write_function(&stream, "(a%d like '::%%' or a%d = '' or a%d like '%q%%')%q",
 										  x + 1, x + 1, x + 1, switch_str_nil(argv[x]), x == argc - 1 ? "" : " and ");
 				} else {
@@ -756,7 +857,8 @@ SWITCH_DECLARE(unsigned char) switch_console_complete(const char *line, const ch
 			}
 		}
 
-		stream.write_function(&stream, " and hostname='%s'", switch_core_get_variable("hostname"));
+		stream.write_function(&stream, " and hostname='%s' order by a%d", switch_core_get_hostname(), h.words + 1);
+
 		switch_cache_db_execute_sql_callback(db, stream.data, comp_callback, &h, &errmsg);
 
 		if (errmsg) {
@@ -799,7 +901,7 @@ SWITCH_DECLARE(unsigned char) switch_console_complete(const char *line, const ch
 			switch_xml_set_txt_d(x_write, h.partial);
 		}
 	}
-#ifdef SWITCH_HAVE_LIBEDIT
+#ifdef HAVE_LIBEDIT
 	if (h.out) {
 		if (h.hits == 1 && !zstr(h.last)) {
 			el_deletestr(el, h.len);
@@ -842,7 +944,7 @@ SWITCH_DECLARE(unsigned char) switch_console_complete(const char *line, const ch
 }
 
 
-
+#if defined(HAVE_LIBEDIT) || defined(_MSC_VER)
 /*
  * If a fnkey is configured then process the command
  */
@@ -850,7 +952,7 @@ static unsigned char console_fnkey_pressed(int i)
 {
 	char *c, *cmd;
 
-	assert((i > 0) && (i <= 12));
+	switch_assert((i > 0) && (i <= 12));
 
 	c = console_fnkeys[i - 1];
 
@@ -868,17 +970,18 @@ static unsigned char console_fnkey_pressed(int i)
 
 	return CC_REDISPLAY;
 }
+#endif
 
 SWITCH_DECLARE(void) switch_console_save_history(void)
 {
-#ifdef SWITCH_HAVE_LIBEDIT
+#ifdef HAVE_LIBEDIT
 	history(myhistory, &ev, H_SAVE, hfile);
 #else
 	switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_CONSOLE, "NOT IMPLEMENTED!\n");
 #endif
 }
 
-#ifdef SWITCH_HAVE_LIBEDIT
+#ifdef HAVE_LIBEDIT
 static char prompt_str[512] = "";
 
 static unsigned char console_f1key(EditLine * el, int ch)
@@ -934,7 +1037,7 @@ static unsigned char console_f12key(EditLine * el, int ch)
 char *prompt(EditLine * e)
 {
 	if (*prompt_str == '\0') {
-		switch_snprintf(prompt_str, sizeof(prompt_str), "freeswitch@%s> ", hostname);
+		switch_snprintf(prompt_str, sizeof(prompt_str), "freeswitch@%s> ", switch_core_get_switchname());
 	}
 
 	return prompt_str;
@@ -1000,8 +1103,6 @@ SWITCH_DECLARE(void) switch_console_loop(void)
 	switch_threadattr_t *thd_attr = NULL;
 	switch_memory_pool_t *pool;
 
-	gethostname(hostname, sizeof(hostname));
-
 	if (switch_core_new_memory_pool(&pool) != SWITCH_STATUS_SUCCESS) {
 		switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_CRIT, "Pool Failure\n");
 		return;
@@ -1053,6 +1154,9 @@ SWITCH_DECLARE(void) switch_console_loop(void)
 	el_set(el, EL_ADDFN, "ed-complete", "Complete argument", complete);
 	el_set(el, EL_BIND, "^I", "ed-complete", NULL);
 
+	/* "Delete" key. */
+	el_set(el, EL_BIND, "\033[3~", "ed-delete-next-char", NULL);
+
 	myhistory = history_init();
 	if (myhistory == 0) {
 		fprintf(stderr, "history could not be initialized\n");
@@ -1220,6 +1324,22 @@ static int console_bufferInput(char *addchars, int len, char *cmd, int key)
 	if (key == KEY_INSERT) {
 		insertMode = !insertMode;
 	}
+	if (key == KEY_DELETE) {
+		if (iCmdCursor < iCmdBuffer) {
+			int pos;
+			for (pos = iCmdCursor; pos < iCmdBuffer; pos++) {
+				cmd[pos] = cmd[pos + 1];
+			}
+			cmd[pos] = 0;
+			iCmdBuffer--;
+
+			for (pos = iCmdCursor; pos < iCmdBuffer; pos++) {
+				printf("%c", cmd[pos]);
+			}
+			printf(" ");
+			SetConsoleCursorPosition(hOut, position);
+		}
+	}
 	for (iBuf = 0; iBuf < len; iBuf++) {
 		switch (addchars[iBuf]) {
 		case '\r':
@@ -1326,7 +1446,7 @@ static int console_bufferInput(char *addchars, int len, char *cmd, int key)
 
 static BOOL console_readConsole(HANDLE conIn, char *buf, int len, int *pRed, int *key)
 {
-	DWORD recordIndex, bufferIndex, toRead, red;
+	DWORD recordIndex, bufferIndex, toRead = 0, red;
 	PINPUT_RECORD pInput;
 
 	GetNumberOfConsoleInputEvents(conIn, &toRead);
@@ -1411,6 +1531,9 @@ static BOOL console_readConsole(HANDLE conIn, char *buf, int len, int *pRed, int
 			if (keyEvent.wVirtualKeyCode == 45 && keyEvent.wVirtualScanCode == 82) {
 				*key = KEY_INSERT;
 			}
+			if (keyEvent.wVirtualKeyCode == 46 && keyEvent.wVirtualScanCode == 83) {
+				*key = KEY_DELETE;
+			}
 			while (keyEvent.wRepeatCount && keyEvent.uChar.AsciiChar) {
 				buf[bufferIndex] = keyEvent.uChar.AsciiChar;
 				if (buf[bufferIndex] == '\r') {
@@ -1436,14 +1559,14 @@ SWITCH_DECLARE(void) switch_console_loop(void)
 #ifndef _MSC_VER
 	int x = 0;
 #else
-	char keys[80];
+	char keys[CMD_BUFLEN];
 #endif
 
 	/* Load/Init the config first */
 	console_xml_config();
-	gethostname(hostname, sizeof(hostname));
+
 #ifdef _MSC_VER
-	sprintf(cmd, "\nfreeswitch@%s> ", hostname);
+	sprintf(cmd, "\nfreeswitch@%s> ", switch_core_get_switchname());
 	console_bufferInput(0, 0, cmd, PROMPT_OP);
 	memset(cmd, 0, sizeof(cmd));
 #endif
@@ -1464,18 +1587,19 @@ SWITCH_DECLARE(void) switch_console_loop(void)
 		}
 
 		if (activity) {
-			switch_log_printf(SWITCH_CHANNEL_LOG_CLEAN, SWITCH_LOG_CONSOLE, "\nfreeswitch@%s> ", hostname);
+			switch_log_printf(SWITCH_CHANNEL_LOG_CLEAN, SWITCH_LOG_CONSOLE, "\nfreeswitch@%s> ", switch_core_get_switchname());
 		}
 #ifdef _MSC_VER
 		activity = 0;
 		if (console_readConsole(stdinHandle, keys, (int) sizeof(keys), &read, &key)) {
 			if (console_bufferInput(keys, read, cmd, key)) {
 				if (!strcmp(cmd, "Empty")) {
-					cmd[0] = '\n';
-					cmd[1] = 0;
+					cmd[0] = 0;
 				}
 				activity = 1;
-				running = switch_console_process(cmd);
+				if (cmd[0]) {
+					running = switch_console_process(cmd);
+				}
 				memset(cmd, 0, sizeof(cmd));
 			}
 		}
@@ -1539,7 +1663,10 @@ static struct {
 SWITCH_DECLARE(switch_status_t) switch_console_init(switch_memory_pool_t *pool)
 {
 	switch_mutex_init(&globals.func_mutex, SWITCH_MUTEX_NESTED, pool);
-	switch_core_hash_init(&globals.func_hash, pool);
+	switch_core_hash_init(&globals.func_hash);
+	switch_console_add_complete_func("::console::list_available_modules", (switch_console_complete_callback_t) switch_console_list_available_modules);
+	switch_console_add_complete_func("::console::list_loaded_modules", (switch_console_complete_callback_t) switch_console_list_loaded_modules);
+	switch_console_add_complete_func("::console::list_interfaces", (switch_console_complete_callback_t) switch_console_list_interfaces);
 	switch_console_add_complete_func("::console::list_uuid", (switch_console_complete_callback_t) switch_console_list_uuid);
 	return SWITCH_STATUS_SUCCESS;
 }
@@ -1610,6 +1737,7 @@ SWITCH_DECLARE(void) switch_console_sort_matches(switch_console_callback_match_t
 		sort[3] = sort[2] ? sort[2]->next : NULL;
 
 		for (j = 1; j <= (matches->count - i); j++) {
+			switch_assert(sort[1] && sort[2]);
 			if (strcmp(sort[1]->val, sort[2]->val) > 0) {
 				sort[1]->next = sort[3];
 				sort[2]->next = sort[1];
@@ -1649,6 +1777,20 @@ SWITCH_DECLARE(void) switch_console_sort_matches(switch_console_callback_match_t
 	}
 }
 
+SWITCH_DECLARE(void) switch_console_push_match_unique(switch_console_callback_match_t **matches, const char *new_val)
+{
+	/* Ignore the entry if it is already in the list */
+	if (*matches) {
+		switch_console_callback_match_node_t *node;
+
+		for(node = (*matches)->head; node; node = node->next) {
+			if (!strcasecmp(node->val, new_val)) return;
+		}
+	}
+
+	switch_console_push_match(matches, new_val);
+}
+
 SWITCH_DECLARE(void) switch_console_push_match(switch_console_callback_match_t **matches, const char *new_val)
 {
 	switch_console_callback_match_node_t *match;
@@ -1695,76 +1837,75 @@ SWITCH_DECLARE(switch_status_t) switch_console_set_complete(const char *string)
 	char *mydata = NULL, *argv[11] = { 0 };
 	int argc, x;
 	switch_status_t status = SWITCH_STATUS_FALSE;
+	switch_core_flag_t cflags = switch_core_flags();
+
+	if (!(cflags & SCF_USE_SQL)) {
+		return SWITCH_STATUS_FALSE;
+	}
 
 	if (string && (mydata = strdup(string))) {
 		if ((argc = switch_separate_string(mydata, ' ', argv, (sizeof(argv) / sizeof(argv[0]))))) {
-			switch_cache_db_handle_t *db = NULL;
 			switch_stream_handle_t mystream = { 0 };
 			SWITCH_STANDARD_STREAM(mystream);
 
-
-			if (switch_core_db_handle(&db) != SWITCH_STATUS_SUCCESS) {
-				switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Database Error\n");
-				free(mystream.data);
-				free(mydata);
-				return SWITCH_STATUS_FALSE;
-			}
-
-
 			if (!strcasecmp(argv[0], "stickyadd")) {
-				mystream.write_function(&mystream, "insert into complete values (1,");
+				mystream.write_function(&mystream, "insert into complete (sticky, a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, hostname) values (1,");
 				for (x = 0; x < 10; x++) {
 					if (argv[x + 1] && !strcasecmp(argv[x + 1], "_any_")) {
 						mystream.write_function(&mystream, "%s", "'', ");
 					} else {
-						if (db->type == SCDB_TYPE_CORE_DB) {
+						if (switch_core_dbtype() == SCDB_TYPE_CORE_DB) {
 							mystream.write_function(&mystream, "'%q', ", switch_str_nil(argv[x + 1]));
 						} else {
 							mystream.write_function(&mystream, "'%w', ", switch_str_nil(argv[x + 1]));
 						}
 					}
 				}
-				mystream.write_function(&mystream, " '%s')", switch_core_get_variable("hostname"));
-				switch_cache_db_persistant_execute(db, mystream.data, 5);
+				mystream.write_function(&mystream, " '%s')", switch_core_get_hostname());
+				switch_core_sql_exec(mystream.data);
 				status = SWITCH_STATUS_SUCCESS;
 			} else if (!strcasecmp(argv[0], "add")) {
-				mystream.write_function(&mystream, "insert into complete values (0,");
+				mystream.write_function(&mystream, "insert into complete (sticky, a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, hostname) values (0,");
 				for (x = 0; x < 10; x++) {
 					if (argv[x + 1] && !strcasecmp(argv[x + 1], "_any_")) {
 						mystream.write_function(&mystream, "%s", "'', ");
 					} else {
-						if (db->type == SCDB_TYPE_CORE_DB) {
+						if (switch_core_dbtype() == SCDB_TYPE_CORE_DB) {
 							mystream.write_function(&mystream, "'%q', ", switch_str_nil(argv[x + 1]));
 						} else {
 							mystream.write_function(&mystream, "'%w', ", switch_str_nil(argv[x + 1]));
 						}
 					}
 				}
-				mystream.write_function(&mystream, " '%s')", switch_core_get_variable("hostname"));
+				mystream.write_function(&mystream, " '%s')", switch_core_get_hostname());
 
-				switch_cache_db_persistant_execute(db, mystream.data, 5);
+				switch_core_sql_exec(mystream.data);
 				status = SWITCH_STATUS_SUCCESS;
 			} else if (!strcasecmp(argv[0], "del")) {
 				char *what = argv[1];
-				if (!strcasecmp(what, "*")) {
-					switch_cache_db_persistant_execute(db, "delete from complete", 1);
+				if (zstr(what)) {
+					switch_safe_free(mystream.data);
+					switch_safe_free(mydata);
+					return SWITCH_STATUS_FALSE;
+				} else if (!strcasecmp(what, "*")) {
+					mystream.write_function(&mystream, "delete from complete where hostname='%q'", switch_core_get_hostname());
+					switch_core_sql_exec(mystream.data);
 				} else {
 					mystream.write_function(&mystream, "delete from complete where ");
 					for (x = 0; x < argc - 1; x++) {
-						if (db->type == SCDB_TYPE_CORE_DB) {
+						if (switch_core_dbtype() == SCDB_TYPE_CORE_DB) {
 							mystream.write_function(&mystream, "a%d = '%q'%q", x + 1, switch_str_nil(argv[x + 1]), x == argc - 2 ? "" : " and ");
 						} else {
 							mystream.write_function(&mystream, "a%d = '%w'%w", x + 1, switch_str_nil(argv[x + 1]), x == argc - 2 ? "" : " and ");
 						}
 					}
-					mystream.write_function(&mystream, " and hostname='%s'", switch_core_get_variable("hostname"));
-					switch_cache_db_persistant_execute(db, mystream.data, 1);
+					mystream.write_function(&mystream, " and hostname='%q'", switch_core_get_hostname());
+					switch_core_sql_exec(mystream.data);
 				}
 				status = SWITCH_STATUS_SUCCESS;
 			}
 
 			switch_safe_free(mystream.data);
-			switch_cache_db_release_db_handle(&db);
 		}
 	}
 
@@ -1786,45 +1927,50 @@ SWITCH_DECLARE(switch_status_t) switch_console_set_alias(const char *string)
 			switch_cache_db_handle_t *db = NULL;
 			char *sql = NULL;
 
+			if (argc > 2 && !strcmp(argv[1], argv[2])) {
+				switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Alias and command cannot be the same, this will cause loop!\n");
+				return SWITCH_STATUS_FALSE;
+			}
+
 			if (switch_core_db_handle(&db) != SWITCH_STATUS_SUCCESS) {
 				switch_log_printf(SWITCH_CHANNEL_LOG, SWITCH_LOG_ERROR, "Database Error\n");
 				free(mydata);
 				return SWITCH_STATUS_FALSE;
 			}
-			
+
 			if (!strcasecmp(argv[0], "stickyadd") && argc == 3) {
-				sql = switch_mprintf("delete from aliases where alias='%q' and hostname='%q'", argv[1], switch_core_get_variable("hostname"));
+				sql = switch_mprintf("delete from aliases where alias='%q' and hostname='%q'", argv[1], switch_core_get_switchname());
 				switch_cache_db_persistant_execute(db, sql, 5);
 				switch_safe_free(sql);
-				if (db->type == SCDB_TYPE_CORE_DB) {
+				if (switch_cache_db_get_type(db) == SCDB_TYPE_CORE_DB) {
 					sql = switch_mprintf("insert into aliases (sticky, alias, command, hostname) values (1, '%q','%q','%q')",
-										 argv[1], argv[2], switch_core_get_variable("hostname"));
+										 argv[1], argv[2], switch_core_get_switchname());
 				} else {
 					sql = switch_mprintf("insert into aliases (sticky, alias, command, hostname) values (1, '%w','%w','%w')",
-										 argv[1], argv[2], switch_core_get_variable("hostname"));
+										 argv[1], argv[2], switch_core_get_switchname());
 				}
 				switch_cache_db_persistant_execute(db, sql, 5);
 				status = SWITCH_STATUS_SUCCESS;
 			} else if (!strcasecmp(argv[0], "add") && argc == 3) {
-				sql = switch_mprintf("delete from aliases where alias='%q' and hostname='%q'", argv[1], switch_core_get_variable("hostname"));
+				sql = switch_mprintf("delete from aliases where alias='%q' and hostname='%q'", argv[1], switch_core_get_switchname());
 				switch_cache_db_persistant_execute(db, sql, 5);
 				switch_safe_free(sql);
-				if (db->type == SCDB_TYPE_CORE_DB) {
+				if (switch_cache_db_get_type(db) == SCDB_TYPE_CORE_DB) {
 					sql = switch_mprintf("insert into aliases (sticky, alias, command, hostname) values (0, '%q','%q','%q')",
-										 argv[1], argv[2], switch_core_get_variable("hostname"));
+										 argv[1], argv[2], switch_core_get_switchname());
 				} else {
 					sql = switch_mprintf("insert into aliases (sticky, alias, command, hostname) values (0, '%w','%w','%w')",
-										 argv[1], argv[2], switch_core_get_variable("hostname"));
+										 argv[1], argv[2], switch_core_get_switchname());
 				}
 				switch_cache_db_persistant_execute(db, sql, 5);
 				status = SWITCH_STATUS_SUCCESS;
 			} else if (!strcasecmp(argv[0], "del") && argc == 2) {
 				char *what = argv[1];
 				if (!strcasecmp(what, "*")) {
-					sql = switch_mprintf("delete from aliases where hostname='%q'", switch_core_get_variable("hostname"));
+					sql = switch_mprintf("delete from aliases where hostname='%q'", switch_core_get_switchname());
 					switch_cache_db_persistant_execute(db, sql, 1);
 				} else {
-					sql = switch_mprintf("delete from aliases where alias='%q' and hostname='%q'", argv[1], switch_core_get_variable("hostname"));
+					sql = switch_mprintf("delete from aliases where alias='%q' and hostname='%q'", argv[1], switch_core_get_switchname());
 					switch_cache_db_persistant_execute(db, sql, 5);
 				}
 				status = SWITCH_STATUS_SUCCESS;
@@ -1851,5 +1997,5 @@ SWITCH_DECLARE(switch_status_t) switch_console_set_alias(const char *string)
  * c-basic-offset:4
  * End:
  * For VIM:
- * vim:set softtabstop=4 shiftwidth=4 tabstop=4:
+ * vim:set softtabstop=4 shiftwidth=4 tabstop=4 noet:
  */
